@@ -98,6 +98,7 @@ public class ChannelNioSocket extends JkHandler
     private boolean nioIsBroken = false;
     private Selector selector = null;
     private int bufferSize = 8*1024;
+    private int packetSize = 8*1024;
 
     private long requestCount=0;
     
@@ -159,6 +160,16 @@ public class ChannelNioSocket extends JkHandler
         return bufferSize;
     }
 
+    public void setPacketSize(int ps) {
+        if(ps < 8*1024) {
+            ps = 8*1024;
+        }
+        packetSize = ps;
+    }
+
+    public int getPacketSize() {
+        return packetSize;
+    }
 
     /**
      * jmx:managed-attribute description="Bind on a specified address" access="READ_WRITE"
@@ -791,7 +802,7 @@ public class ChannelNioSocket extends JkHandler
 
     protected class SocketConnection implements ThreadPoolRunnable {
         MsgContext ep;
-        MsgAjp recv = new MsgAjp();
+        MsgAjp recv = new MsgAjp(packetSize);
         boolean inProgress = false;
 
         SocketConnection(MsgContext ep) {
