@@ -15,16 +15,16 @@
  */
 
 /***************************************************************************
- * Description: Apache 1.3 plugin for Jakarta/Tomcat                       *
+ * Description: Apache 1.3 plugin for Tomcat                                *
  *              See ../common/jk_service.h for general mod_jk info         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  *              Dan Milstein <danmil@shore.net>                            *
  *              Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 301032 $                                          *
+ * Version:     $Revision: 440636 $                                          *
  ***************************************************************************/
 
 /*
- * mod_jk: keeps all servlet/jakarta related ramblings together.
+ * mod_jk: keeps all servlet related ramblings together.
  */
 
 /* #include "ap_config.h" */
@@ -40,7 +40,7 @@
 #include "http_conf_globals.h"
 
 /*
- * Jakarta (jk_) include files
+ * jk_ include files
  */
 #ifdef NETWARE
 #define _SYS_TYPES_H_
@@ -60,9 +60,11 @@
 #include "jk_ajp13.h"
 #include "jk_shm.h"
 
-#define JK_WORKER_ID        ("jakarta.worker")
+#define JK_NOTE_WORKER_NAME         ("JK_WORKER_NAME")
+#define JK_NOTE_WORKER_TYPE         ("JK_WORKER_TYPE")
+#define JK_NOTE_REQUEST_DURATION    ("JK_REQUEST_DURATION")
+#define JK_NOTE_WORKER_ROUTE        ("JK_WORKER_ROUTE")
 #define JK_HANDLER          ("jakarta-servlet")
-#define JK_DURATION         ("jakarta.worker.duration")
 #define JK_MAGIC_TYPE       ("application/x-jakarta-servlet")
 #define NULL_FOR_EMPTY(x)   ((x && !strlen(x)) ? NULL : x)
 #define STRNULL_FOR_NULL(x) ((x) ? (x) : "(null)")
@@ -82,6 +84,14 @@ extern __declspec(dllimport) module dir_module;
 extern module dir_module;
 #endif
 
+static int xfer_flags = (O_WRONLY | O_APPEND | O_CREAT);
+#if defined(OS2) || defined(WIN32) || defined(NETWARE)
+/* OS/2 dosen't support users and groups */
+static mode_t xfer_mode = (S_IREAD | S_IWRITE);
+#else
+static mode_t xfer_mode = (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+#endif
+
 /*
  * Configuration object for the mod_jk module.
  */
@@ -92,6 +102,7 @@ typedef struct
      * Log stuff
      */
     char *log_file;
+    int log_fd;
     int log_level;
     jk_logger_t *log;
 
@@ -192,6 +203,12 @@ static int JK_METHOD ws_read(jk_ws_service_t *s,
                              void *b, unsigned l, unsigned *a);
 
 static int JK_METHOD ws_write(jk_ws_service_t *s, const void *b, unsigned l);
+
+static void JK_METHOD ws_add_log_items(jk_ws_service_t *s,
+                                       const char *const *log_names,
+                                       const char *const *log_values,
+                                       unsigned num_of_log_items);
+
 /* srevilak - new function prototypes */
 static void jk_server_cleanup(void *data);
 static void jk_generic_cleanup(server_rec * s);
@@ -298,6 +315,8 @@ static int JK_METHOD ws_read(jk_ws_service_t *s,
             else {
                 *actually_read = (unsigned)rv;
             }
+            /* reset timeout after successful read */
+            ap_reset_timeout(p->r);
             return JK_TRUE;
         }
     }
@@ -335,6 +354,9 @@ static int JK_METHOD ws_write(jk_ws_service_t *s, const void *b, unsigned len)
             int r = 0;
 
             if (!p->response_started) {
+                if (main_log)
+                    jk_log(main_log, JK_LOG_INFO,
+                           "Write without start, starting with defaults");
                 if (!s->start_response(s, 200, NULL, NULL, NULL, 0)) {
                     return JK_FALSE;
                 }
@@ -364,6 +386,8 @@ static int JK_METHOD ws_write(jk_ws_service_t *s, const void *b, unsigned len)
                 }
 
             }
+            if (p->r->connection->aborted)
+                return JK_FALSE;
         }
 
         return JK_TRUE;
@@ -371,6 +395,22 @@ static int JK_METHOD ws_write(jk_ws_service_t *s, const void *b, unsigned len)
     return JK_FALSE;
 }
 
+static void JK_METHOD ws_add_log_items(jk_ws_service_t *s,
+                                       const char *const *log_names,
+                                       const char *const *log_values,
+                                       unsigned num_of_log_items)
+{
+    unsigned h;
+    apache_private_data_t *p = s->ws_private;
+    request_rec *r = p->r;
+
+    for (h = 0; h < num_of_log_items; h++) {
+        if (log_names[h] && log_values[h]) {
+            ap_table_setn(r->notes, log_names[h], log_values[h]);
+        }
+    }
+}
+
 /* ====================================================================== */
 /* Utility functions                                                      */
 /* ====================================================================== */
@@ -445,6 +485,7 @@ static int init_ws_service(apache_private_data_t * private_data,
     s->read = ws_read;
     s->write = ws_write;
     s->flush = ws_flush;
+    s->add_log_items = ws_add_log_items;
 
     /* Clear RECO status */
     s->reco_status = RECO_NONE;
@@ -457,7 +498,7 @@ static int init_ws_service(apache_private_data_t * private_data,
         (char *)ap_get_remote_host(r->connection, r->per_dir_config,
                                    REMOTE_HOST);
     s->remote_host = NULL_FOR_EMPTY(s->remote_host);
-    
+
     if (conf->options & JK_OPT_FWDLOCAL)
         s->remote_addr = NULL_FOR_EMPTY(r->connection->local_ip);
     else
@@ -467,14 +508,20 @@ static int init_ws_service(apache_private_data_t * private_data,
         s->flush_packets = 1;
     else
         s->flush_packets = 0;
+
+    if (conf->options & JK_OPT_DISABLEREUSE)
+        s->disable_reuse = 1;
+    else
+        s->disable_reuse = 0;
+
     /* get server name */
     /* s->server_name  = (char *)(r->hostname ? r->hostname : r->server->server_hostname); */
-    /* XXX :  la jk2 */
+    /* XXX : a la jk2 */
     s->server_name = (char *)ap_get_server_name(r);
 
     /* get the real port (otherwise redirect failed) */
     /* s->server_port     = htons( r->connection->local_addr.sin_port ); */
-    /* XXX :  la jk2 */
+    /* XXX : a la jk2 */
     s->server_port = ap_get_server_port(r);
 
     s->server_software = (char *)ap_get_server_version();
@@ -490,7 +537,7 @@ static int init_ws_service(apache_private_data_t * private_data,
      */
     if (JK_IS_DEBUG_LEVEL(conf->log)) {
         jk_log(conf->log, JK_LOG_DEBUG,
-               "Service protocol=%s method=%s host=%s addrr=%s name=%s port=%d auth=%s user=%s laddr=%s raddr=%s",
+               "Service protocol=%s method=%s host=%s addr=%s name=%s port=%d auth=%s user=%s laddr=%s raddr=%s",
                STRNULL_FOR_NULL(s->protocol),
                STRNULL_FOR_NULL(s->method),
                STRNULL_FOR_NULL(s->remote_host),
@@ -880,17 +927,7 @@ static const char *jk_set_log_file(cmd_parms * cmd,
                                                   &jk_module);
 
     /* we need an absolut path */
-    conf->log_file = ap_server_root_relative(cmd->pool, log_file);
-
-#ifdef CHROOTED_APACHE
-    ap_server_strip_chroot(conf->log_file, 0);
-#endif
-
-    if (conf->log_file == log_file)
-        conf->log_file = ap_pstrdup(cmd->pool, log_file);
-
-    if (conf->log_file == NULL)
-        return "JkLogFile file_name invalid";
+    conf->log_file = log_file;
 
     return NULL;
 }
@@ -1072,13 +1109,17 @@ static const char *constant_item(request_rec * dummy, char *stuff)
 
 static const char *log_worker_name(request_rec * r, char *a)
 {
-    return ap_table_get(r->notes, JK_WORKER_ID);
+    return ap_table_get(r->notes, JK_NOTE_WORKER_NAME);
 }
 
+static const char *log_worker_route(request_rec * r, char *a)
+{
+    return ap_table_get(r->notes, JK_NOTE_WORKER_ROUTE);
+}
 
 static const char *log_request_duration(request_rec * r, char *a)
 {
-    return ap_table_get(r->notes, JK_DURATION);
+    return ap_table_get(r->notes, JK_NOTE_REQUEST_DURATION);
 }
 
 static const char *log_request_line(request_rec * r, char *a)
@@ -1189,6 +1230,7 @@ static struct log_item_list
     'm', log_request_method}, {
     'q', log_request_query}, {
     'w', log_worker_name}, {
+    'R', log_worker_route}, {
     '\0'}
 };
 
@@ -1306,9 +1348,6 @@ static array_header *parse_request_log_string(pool * p, const char *s,
         }
     }
 
-    s = "\n";
-    parse_request_log_item(p, (request_log_format_item *) ap_push_array(a),
-                           &s);
     return a;
 }
 
@@ -1519,6 +1558,9 @@ const char *jk_set_options(cmd_parms * cmd, void *dummy, const char *line)
         else if (!strcasecmp(w, "FlushPackets")) {
             opt = JK_OPT_FLUSHPACKETS;
         }
+        else if (!strcasecmp(w, "DisableReuse")) {
+            opt = JK_OPT_DISABLEREUSE;
+        }
         else
             return ap_pstrcat(cmd->pool, "JkOptions: Illegal option '", w,
                               "'", NULL);
@@ -1575,7 +1617,8 @@ static const char *jk_set_worker_property(cmd_parms * cmd,
         (jk_server_conf_t *) ap_get_module_config(s->module_config,
                                                   &jk_module);
 
-    if (jk_map_read_property(conf->worker_properties, line) == JK_FALSE)
+    jk_logger_t *l = conf->log ? conf->log : main_log;
+    if (jk_map_read_property(conf->worker_properties, line, l) == JK_FALSE)
         return ap_pstrcat(cmd->temp_pool, "Invalid JkWorkerProperty ", line);
 
     return NULL;
@@ -1590,7 +1633,7 @@ static const command_rec jk_cmds[] = {
      * servlet requests.
      */
     {"JkWorkersFile", jk_set_worker_file, NULL, RSRC_CONF, TAKE1,
-     "the name of a worker file for the Jakarta servlet containers"},
+     "the name of a worker file for the Tomcat servlet containers"},
 
     /*
      * JkMountFile specifies a full path to the location of the
@@ -1600,7 +1643,7 @@ static const command_rec jk_cmds[] = {
      * to redirect servlet requests.
      */
     {"JkMountFile", jk_set_mount_file, NULL, RSRC_CONF, TAKE1,
-     "the name of a mount file for the Jakarta servlet uri mappings"},
+     "the name of a mount file for the Tomcat servlet uri mappings"},
 
     /*
      * JkAutoMount specifies that the list of handled URLs must be
@@ -1636,24 +1679,24 @@ static const command_rec jk_cmds[] = {
      * JkLogStampFormat specify the time-stamp to be used on log
      */
     {"JkLogFile", jk_set_log_file, NULL, RSRC_CONF, TAKE1,
-     "Full path to the Jakarta mod_jk module log file"},
+     "Full path to the mod_jk module log file"},
     {"JkShmFile", jk_set_shm_file, NULL, RSRC_CONF, TAKE1,
-     "Full path to the Jakarta mod_jk module shared memory file"},
+     "Full path to the mod_jk module shared memory file"},
     {"JkShmSize", jk_set_shm_size, NULL, RSRC_CONF, TAKE1,
      "Size of the shared memory file in KBytes"},
     {"JkLogLevel", jk_set_log_level, NULL, RSRC_CONF, TAKE1,
-     "The Jakarta mod_jk module log level, can be debug, info, request, error, or emerg"},
+     "The mod_jk module log level, can be debug, info, request, error, or emerg"},
     {"JkLogStampFormat", jk_set_log_fmt, NULL, RSRC_CONF, TAKE1,
-     "The Jakarta mod_jk module log format, follow strftime synthax"},
+     "The mod_jk module log format, follow strftime synthax"},
     {"JkRequestLogFormat", jk_set_request_log_format, NULL, RSRC_CONF, TAKE1,
-     "The Jakarta mod_jk module request log format string"},
+     "The mod_jk module request log format string"},
 
     /*
      * Automatically Alias webapp context directories into the Apache
      * document space.
      */
     {"JkAutoAlias", jk_set_auto_alias, NULL, RSRC_CONF, TAKE1,
-     "The Jakarta mod_jk module automatic context apache alias directory"},
+     "The mod_jk module automatic context apache alias directory"},
 
     /*
      * Apache has multiple SSL modules (for example apache_ssl, stronghold
@@ -1715,7 +1758,7 @@ static const command_rec jk_cmds[] = {
 static int jk_handler(request_rec * r)
 {
     /* Retrieve the worker name stored by jk_translate() */
-    const char *worker_name = ap_table_get(r->notes, JK_WORKER_ID);
+    const char *worker_name = ap_table_get(r->notes, JK_NOTE_WORKER_NAME);
     int rc;
 
     if (r->proxyreq) {
@@ -1761,6 +1804,8 @@ static int jk_handler(request_rec * r)
             s.retries = worker->retries;
             s.ws_private = &private_data;
             s.pool = &private_data.p;
+            ap_table_setn(r->notes, JK_NOTE_WORKER_TYPE,
+                          wc_get_name_for_type(worker->type, l));
 #ifndef NO_GETTIMEOFDAY
             if (conf->format != NULL) {
                 gettimeofday(&tv_begin, NULL);
@@ -1790,10 +1835,10 @@ static int jk_handler(request_rec * r)
                         }
                     }
                 }
-#ifndef NO_GETTIMEOFDAY
                 if (conf->format != NULL) {
-                    char *duration = NULL;
+#ifndef NO_GETTIMEOFDAY
                     long micro, seconds;
+                    char *duration = NULL;
                     gettimeofday(&tv_end, NULL);
                     if (tv_end.tv_usec < tv_begin.tv_usec) {
                         tv_end.tv_usec += 1000000;
@@ -1803,10 +1848,12 @@ static int jk_handler(request_rec * r)
                     seconds = tv_end.tv_sec - tv_begin.tv_sec;
                     duration =
                         ap_psprintf(r->pool, "%.1ld.%.6ld", seconds, micro);
-                    ap_table_setn(r->notes, JK_DURATION, duration);
+                    ap_table_setn(r->notes, JK_NOTE_REQUEST_DURATION, duration);
+#endif
+                    if (s.jvm_route && *s.jvm_route)
+                        ap_table_setn(r->notes, JK_NOTE_WORKER_ROUTE, s.jvm_route);
                     request_log_transaction(r, conf);
                 }
-#endif
             }
             else {
                 jk_log(l, JK_LOG_ERROR, "Could not init service"
@@ -1831,7 +1878,7 @@ static int jk_handler(request_rec * r)
                 if (JK_IS_DEBUG_LEVEL(l))
                     jk_log(l, JK_LOG_DEBUG, "Service finished"
                            " with status=%d for worker=%s",
-                           r->status, worker_name); 
+                           r->status, worker_name);
                 JK_TRACE_EXIT(l);
                 return OK;      /* NOT r->status, even if it has changed. */
             }
@@ -1877,7 +1924,8 @@ static void *create_jk_config(ap_pool * p, server_rec * s)
     c->worker_file = NULL;
     c->mount_file = NULL;
     c->log_file = NULL;
-    c->log_level = -1;
+    c->log_fd = -1;
+    c->log_level = JK_LOG_DEF_LEVEL;
     c->log = NULL;
     c->alias_dir = NULL;
     c->format_string = NULL;
@@ -1980,12 +2028,6 @@ static void *merge_jk_config(ap_pool * p, void *basev, void *overridesv)
             ap_overlay_tables(p, overrides->envvars, base->envvars);
     }
 
-    if (overrides->log_file && overrides->log_level >= 0) {
-        if (!jk_open_file_logger
-            (&(overrides->log), overrides->log_file, overrides->log_level)) {
-            overrides->log = NULL;
-        }
-    }
     if (!uri_worker_map_alloc
         (&(overrides->uw_map), overrides->uri_to_context, overrides->log)) {
         jk_error_exit(APLOG_MARK, APLOG_EMERG, overrides->s, p,
@@ -1998,28 +2040,98 @@ static void *merge_jk_config(ap_pool * p, void *basev, void *overridesv)
     return overrides;
 }
 
+static int JK_METHOD jk_log_to_file(jk_logger_t *l,
+                                    int level, const char *what)
+{
+    if (l &&
+        (l->level <= level || level == JK_LOG_REQUEST_LEVEL) &&
+         l->logger_private && what) {
+        file_logger_t *flp = l->logger_private;
+        int log_fd = flp->log_fd;
+        size_t sz = strlen(what);
+        if (log_fd >= 0 && sz) {
+            if ( write(log_fd, what, sz) < 0 )
+            {
+                ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO,
+                             NULL,
+                             "mod_jk: jk_log_to_file %s failed",
+                             what);
+            }
+        }
+
+        return JK_TRUE;
+    }
+
+    return JK_FALSE;
+}
+
+static void open_jk_log(server_rec *s, pool *p)
+{
+    jk_logger_t *jkl;
+    file_logger_t *flp;
+    jk_server_conf_t *conf =
+        (jk_server_conf_t *) ap_get_module_config(s->module_config,
+                                                  &jk_module);
+
+    if (!conf->log_file || conf->log_fd >= 0)
+        return;               /* virtual log shared w/main server */
+
+    if (*conf->log_file == '|') {
+        piped_log *pl;
+
+        pl = ap_open_piped_log(p, conf->log_file + 1);
+        if (pl == NULL) {
+            ap_log_error(APLOG_MARK, APLOG_ERR, s,
+                         "mod_jk: could not open reliable pipe "
+                         "to jk log %s", conf->log_file + 1);
+            exit(1);
+        }
+        conf->log_fd = ap_piped_log_write_fd(pl);
+    }
+    else if (*conf->log_file != '\0') {
+        char *log_file = ap_server_root_relative(p, conf->log_file);
+#if AP_MODULE_MAGIC_AT_LEAST(19990320,14)
+        if ((conf->log_fd = ap_popenf_ex(p, log_file, xfer_flags, xfer_mode, 1))
+             < 0) {
+#else
+        if ((conf->log_fd = ap_popenf(p, log_file, xfer_flags, xfer_mode))
+             < 0) {
+#endif
+            ap_log_error(APLOG_MARK, APLOG_ERR, s,
+                         "mod_jk: could not open JkLog " "file %s", log_file);
+            exit(1);
+        }
+    }
+
+    jkl = (jk_logger_t *)ap_palloc(p, sizeof(jk_logger_t));
+    flp = (file_logger_t *)ap_palloc(p, sizeof(file_logger_t));
+    if (jkl && flp) {
+        jkl->log = jk_log_to_file;
+        jkl->level = conf->log_level;
+        jkl->logger_private = flp;
+        flp->log_fd = conf->log_fd;
+        conf->log = jkl;
+        if (main_log == NULL)
+            main_log = conf->log;
+        return;
+    }
+
+    return;
+}
+
 static void jk_init(server_rec * s, ap_pool * p)
 {
     int rc;
+    server_rec *t;
     jk_server_conf_t *conf =
         (jk_server_conf_t *) ap_get_module_config(s->module_config,
                                                   &jk_module);
     jk_map_t *init_map = conf->worker_properties;
 
-    /* Open up log file */
-    if (conf->log_file && conf->log_level >= 0) {
-        if (!jk_open_file_logger
-            (&(conf->log), conf->log_file, conf->log_level)) {
-#ifdef CHROOTED_APACHE
-            conf->log = main_log;
-#else
-            conf->log = NULL;
-#endif
-        }
-        else {
-            main_log = conf->log;
-        }
-    }
+    open_jk_log(s, p);
+    main_log = conf->log;
+    for (t=s; t; t = t->next)
+        open_jk_log(t, p);
 #if !defined(WIN32) && !defined(NETWARE)
     if (!jk_shm_file) {
         jk_shm_file = ap_server_root_relative(p, "logs/jk-runtime-status");
@@ -2029,7 +2141,7 @@ static void jk_init(server_rec * s, ap_pool * p)
                          "Using default %s", jk_shm_file);
     }
 #endif
-    
+
     if ((rc = jk_shm_open(jk_shm_file, jk_shm_size, conf->log)) == 0) {
         if (JK_IS_DEBUG_LEVEL(conf->log))
             jk_log(conf->log, JK_LOG_DEBUG, "Initialized shm:%s",
@@ -2044,7 +2156,7 @@ static void jk_init(server_rec * s, ap_pool * p)
                      "No JkShmFile defined in httpd.conf. "
                      "LoadBalancer will not function properly!");
 #endif
- 
+
     /* SREVILAK -- register cleanup handler to clear resources on restart,
      * to make sure log file gets closed in the parent process  */
     ap_register_cleanup(p, s, jk_server_cleanup, ap_null_cleanup);
@@ -2055,7 +2167,7 @@ jk_log(conf->log, JK_LOG_DEBUG, "default secret key = %s", conf->secret_key);
 for (i = 0; i < jk_map_size(conf->automount); i++)
 {
             char *name = jk_map_name_at(conf->automount, i);
-			jk_log(conf->log, JK_LOG_DEBUG, "worker = %s and virtualhost = %s", name, map_get_string(conf->automount, name, NULL));
+            jk_log(conf->log, JK_LOG_DEBUG, "worker = %s and virtualhost = %s", name, map_get_string(conf->automount, name, NULL));
 }
 }
 */
@@ -2068,7 +2180,7 @@ for (i = 0; i < jk_map_size(conf->automount); i++)
 
     /*if(map_alloc(&init_map)) { */
 
-    if (!jk_map_read_properties(init_map, conf->worker_file, NULL)) {
+    if (!jk_map_read_properties(init_map, conf->worker_file, NULL, conf->log)) {
 
         if (jk_map_size(init_map) == 0) {
             ap_log_error(APLOG_MARK, APLOG_EMERG, NULL,
@@ -2083,6 +2195,10 @@ for (i = 0; i < jk_map_size(conf->automount); i++)
     ap_add_version_component(JK_EXPOSED_VERSION);
 #endif
 
+    if (jk_map_resolve_references(init_map, "worker.", 1, 1, conf->log) == JK_FALSE) {
+        jk_error_exit(APLOG_MARK, APLOG_EMERG, s, p, "Error in resolving configuration references");
+    }
+
     /* we add the URI->WORKER MAP since workers using AJP14 will feed it */
     worker_env.uri_to_worker = conf->uw_map;
     worker_env.virtual = "*";       /* for now */
@@ -2129,7 +2245,7 @@ static int jk_translate(request_rec * r)
                 r->prev && r->prev->handler &&
                 !strcmp(r->prev->handler, JK_HANDLER) && clean_uri &&
                 strlen(clean_uri) && clean_uri[strlen(clean_uri) - 1] == '/') {
-                
+
                 if (worker_env.num_of_workers) {
                     /* Nothing here to do but assign the first worker since we
                      * already tried mapping and it didn't work out */
@@ -2142,7 +2258,7 @@ static int jk_translate(request_rec * r)
 
             if (worker) {
                 r->handler = ap_pstrdup(r->pool, JK_HANDLER);
-                ap_table_setn(r->notes, JK_WORKER_ID, worker);
+                ap_table_setn(r->notes, JK_NOTE_WORKER_NAME, worker);
             }
             else if (conf->alias_dir != NULL) {
                 /* Automatically map uri to a context static file */
@@ -2240,7 +2356,7 @@ static int jk_fixups(request_rec * r)
     if (r->main) {
         jk_server_conf_t *conf = (jk_server_conf_t *)
             ap_get_module_config(r->server->module_config, &jk_module);
-        char *worker = (char *)ap_table_get(r->notes, JK_WORKER_ID);
+        char *worker = (char *)ap_table_get(r->notes, JK_NOTE_WORKER_NAME);
 
         /* Only if we have no worker and ForwardDirectories is set */
         if (!worker && (conf->options & JK_OPT_FWDDIRS)) {
@@ -2314,14 +2430,11 @@ static void jk_generic_cleanup(server_rec * s)
                                                       &jk_module);
 
         if (conf) {
-            wc_close(conf->log);
-            uri_worker_map_free(&(conf->uw_map), conf->log);
+            wc_close(NULL);
+            uri_worker_map_free(&(conf->uw_map), NULL);
             jk_map_free(&(conf->uri_to_context));
             jk_map_free(&(conf->worker_properties));
             jk_map_free(&(conf->automount));
-            if (conf->log)
-                jk_close_file_logger(&(conf->log));
-            conf->log = NULL;
         }
         tmp = tmp->next;
     }
