@@ -19,7 +19,7 @@
  *              This worker is deprecated                                  *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Based on:    jserv_ajpv12.c from Jserv                                  *
- * Version:     $Revision: 300520 $                                          *
+ * Version:     $Revision: 434177 $                                          *
  ***************************************************************************/
 
 #include "jk_ajp12_worker.h"
@@ -51,7 +51,7 @@ struct ajp12_endpoint
 {
     ajp12_worker_t *worker;
 
-    int sd;
+    jk_sock_t sd;
     jk_sockbuf_t sb;
 
     jk_endpoint_t endpoint;
@@ -87,45 +87,50 @@ static int JK_METHOD service(jk_endpoint_t *e,
                              jk_ws_service_t *s,
                              jk_logger_t *l, int *is_error)
 {
-    jk_log(l, JK_LOG_DEBUG, "Into jk_endpoint_t::service");
+    ajp12_endpoint_t *p = e->endpoint_private;
+    unsigned int attempt;
+    int rc = -1;
+    /*
+     * AJP12 protocol is not recoverable.
+     */
 
-    if (e && e->endpoint_private && s && is_error) {
-        ajp12_endpoint_t *p = e->endpoint_private;
-        unsigned int attempt;
-        /*
-         * AJP12 protocol is not recoverable.
-         */
+    JK_TRACE_ENTER(l);
+ 
+    if (is_error)
         *is_error = JK_HTTP_SERVER_ERROR;
+    if (!e || !e->endpoint_private || !s || !is_error) {
+        JK_LOG_NULL_PARAMS(l);
+        JK_TRACE_EXIT(l);
+        return JK_FALSE;
+    }
 
-        for (attempt = 0; attempt < p->worker->connect_retry_attempts;
-             attempt++) {
-            p->sd =
-                jk_open_socket(&p->worker->worker_inet_addr,
-                               JK_FALSE, -1, 0, l);
+    for (attempt = 0; attempt < p->worker->connect_retry_attempts;
+         attempt++) {
+        p->sd =
+            jk_open_socket(&p->worker->worker_inet_addr,
+                           JK_FALSE, -1, 0, l);
 
-            jk_log(l, JK_LOG_DEBUG, "In jk_endpoint_t::service, sd = %d",
-                   p->sd);
-            if (p->sd >= 0) {
-                break;
-            }
-        }
-        if (p->sd >= 0) {
-
-            jk_sb_open(&p->sb, p->sd);
-            if (ajpv12_handle_request(p, s, l)) {
-                jk_log(l, JK_LOG_DEBUG,
-                       "In jk_endpoint_t::service, sent request");
-                return ajpv12_handle_response(p, s, l);
-            }
-        }
-        jk_log(l, JK_LOG_ERROR, "In jk_endpoint_t::service, Error sd = %d",
+        jk_log(l, JK_LOG_DEBUG, "In jk_endpoint_t::service, sd = %d",
                p->sd);
+        if (IS_VALID_SOCKET(p->sd)) {
+            break;
+        }
     }
-    else {
-        jk_log(l, JK_LOG_ERROR,
-               "In jk_endpoint_t::service, NULL parameters");
-    }
+    if (IS_VALID_SOCKET(p->sd)) {
 
+        jk_sb_open(&p->sb, p->sd);
+        if (ajpv12_handle_request(p, s, l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "In jk_endpoint_t::service, sent request");
+            rc = ajpv12_handle_response(p, s, l);
+            JK_TRACE_EXIT(l);
+            return rc;
+        }
+    }
+    jk_log(l, JK_LOG_ERROR, "In jk_endpoint_t::service, Error sd = %d",
+           p->sd);
+
+    JK_TRACE_EXIT(l);
     return JK_FALSE;
 }
 
@@ -134,7 +139,7 @@ static int JK_METHOD done(jk_endpoint_t **e, jk_logger_t *l)
     jk_log(l, JK_LOG_DEBUG, "Into jk_endpoint_t::done");
     if (e && *e && (*e)->endpoint_private) {
         ajp12_endpoint_t *p = (*e)->endpoint_private;
-        if (p->sd > 0) {
+        if (IS_VALID_SOCKET(p->sd)) {
             jk_close_socket(p->sd);
         }
         free(p);
@@ -201,7 +206,7 @@ static int JK_METHOD get_endpoint(jk_worker_t *pThis,
         ajp12_endpoint_t *p =
             (ajp12_endpoint_t *) malloc(sizeof(ajp12_endpoint_t));
         if (p) {
-            p->sd = -1;
+            p->sd = JK_INVALID_SOCKET;
             p->worker = pThis->worker_private;
             p->endpoint.endpoint_private = p;
             p->endpoint.service = service;
@@ -313,7 +318,7 @@ static int ajpv12_sendstring(ajp12_endpoint_t * p, const char *buffer)
 {
     int bufferlen;
 
-    if (buffer && (bufferlen = strlen(buffer))) {
+    if (buffer && (bufferlen = (int)strlen(buffer))) {
 #if defined(AS400) || defined(_OSD_POSIX)
         char buf[2048];
         if (bufferlen < 2048) {
