@@ -22,7 +22,7 @@
  * @package jk_connect
  * @author      Gal Shachor <shachor@il.ibm.com>
  * @author      Mladen Turk <mturk@apache.org>
- * @version     $Revision: 386629 $
+ * @version     $Revision: 440792 $
  */
 
 
@@ -37,6 +37,11 @@
 static apr_pool_t *jk_apr_pool = NULL;
 #endif
 
+#ifdef HAVE_SYS_FILIO_H
+/* FIONREAD on Solaris et al. */
+#include <sys/filio.h>
+#endif
+
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
 #define JK_IS_SOCKET_ERROR(x) ((x) == SOCKET_ERROR)
 #define JK_GET_SOCKET_ERRNO() errno = WSAGetLastError() - WSABASEERR
@@ -52,7 +57,7 @@ typedef char* SET_TYPE;
 typedef const char* SET_TYPE;
 #endif
 
-static int soblock(int sd)
+static int soblock(jk_sock_t sd)
 {
 /* BeOS uses setsockopt at present for non blocking... */
 #ifndef WIN32
@@ -81,7 +86,7 @@ static int soblock(int sd)
     return 0;
 }
 
-static int sononblock(int sd)
+static int sononblock(jk_sock_t sd)
 {
 #ifndef WIN32
     int fd_flags;
@@ -111,7 +116,7 @@ static int sononblock(int sd)
 
 #if defined (WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
 /* WIN32 implementation */
-static int nb_connect(int sock, struct sockaddr *addr, int timeout)
+static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
 {
     int rc;
     if (timeout < 1)
@@ -136,7 +141,7 @@ static int nb_connect(int sock, struct sockaddr *addr, int timeout)
 
         tv.tv_sec  = timeout;
         tv.tv_usec = 0;
-        rc = select(FD_SETSIZE+1, NULL, &wfdset, &efdset, &tv);
+        rc = select((int)sock + 1, NULL, &wfdset, &efdset, &tv);
         if (rc == SOCKET_ERROR || rc == 0) {
             rc = WSAGetLastError();
             soblock(sock);
@@ -146,7 +151,7 @@ static int nb_connect(int sock, struct sockaddr *addr, int timeout)
         /* Evaluate the efdset */
         if (FD_ISSET(sock, &efdset)) {
             /* The connect failed. */
-            int rclen = sizeof(rc);
+            int rclen = (int)sizeof(rc);
             if (getsockopt(sock, SOL_SOCKET, SO_ERROR, (char*) &rc, &rclen))
                 rc = 0;
             soblock(sock);
@@ -161,7 +166,7 @@ static int nb_connect(int sock, struct sockaddr *addr, int timeout)
 
 #elif !defined(NETWARE)
 /* POSIX implementation */
-static int nb_connect(int sock, struct sockaddr *addr, int timeout)
+static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
 {
     int rc = 0;
 
@@ -177,13 +182,13 @@ static int nb_connect(int sock, struct sockaddr *addr, int timeout)
                    && (timeout > 0)) {
         fd_set wfdset;
         struct timeval tv;
-        unsigned int rclen = sizeof(rc);
+        socklen_t rclen = (socklen_t)sizeof(rc);
 
         FD_ZERO(&wfdset);
         FD_SET(sock, &wfdset);
         tv.tv_sec  = timeout;
         tv.tv_usec = 0;
-        rc = select(sock+1, NULL, &wfdset, NULL, &tv);
+        rc = select(sock + 1, NULL, &wfdset, NULL, &tv);
         if (rc <= 0) {
             /* Save errno */
             int err = errno;
@@ -210,7 +215,7 @@ static int nb_connect(int sock, struct sockaddr *addr, int timeout)
 }
 #else
 /* NETWARE implementation - blocking for now */
-static int nb_connect(int sock, struct sockaddr *addr, int timeout)
+static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
 {
     return connect(sock, addr, sizeof(struct sockaddr_in));
 }
@@ -294,11 +299,11 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
 
 /** connect to Tomcat */
 
-int jk_open_socket(struct sockaddr_in *addr, int keepalive,
-                   int timeout, int sock_buf, jk_logger_t *l)
+jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
+                         int timeout, int sock_buf, jk_logger_t *l)
 {
     char buf[32];
-    int sock;
+    jk_sock_t sock;
     int set = 1;
     int ret = 0;
 #ifdef SO_LINGER
@@ -308,12 +313,13 @@ int jk_open_socket(struct sockaddr_in *addr, int keepalive,
     JK_TRACE_ENTER(l);
 
     sock = socket(AF_INET, SOCK_STREAM, 0);
-    if (sock < 0) {
+    if (!IS_VALID_SOCKET(sock)) {
         JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
                "socket() failed with errno=%d", errno);
         JK_TRACE_EXIT(l);
-        return -1;
+        return JK_INVALID_SOCKET;
+;
     }
     /* Disable Nagle algorithm */
     if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (SET_TYPE)&set,
@@ -322,7 +328,7 @@ int jk_open_socket(struct sockaddr_in *addr, int keepalive,
                 "failed setting TCP_NODELAY with errno=%d", errno);
         jk_close_socket(sock);
         JK_TRACE_EXIT(l);
-        return -1;
+        return JK_INVALID_SOCKET;
     }
     if (JK_IS_DEBUG_LEVEL(l))
         jk_log(l, JK_LOG_DEBUG,
@@ -335,7 +341,7 @@ int jk_open_socket(struct sockaddr_in *addr, int keepalive,
                    "failed setting SO_KEEPALIVE with errno=%d", errno);
             jk_close_socket(sock);
             JK_TRACE_EXIT(l);
-            return -1;
+            return JK_INVALID_SOCKET;
         }
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
@@ -352,7 +358,7 @@ int jk_open_socket(struct sockaddr_in *addr, int keepalive,
                     "failed setting SO_SNDBUF with errno=%d", errno);
             jk_close_socket(sock);
             JK_TRACE_EXIT(l);
-            return -1;
+            return JK_INVALID_SOCKET;
         }
         set = sock_buf;
         /* Set socket receive buffer size */
@@ -363,7 +369,7 @@ int jk_open_socket(struct sockaddr_in *addr, int keepalive,
                     "failed setting SO_RCVBUF with errno=%d", errno);
             jk_close_socket(sock);
             JK_TRACE_EXIT(l);
-            return -1;
+            return JK_INVALID_SOCKET;
         }
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
@@ -405,7 +411,7 @@ int jk_open_socket(struct sockaddr_in *addr, int keepalive,
                 "failed setting SO_NOSIGPIPE with errno=%d", errno);
         jk_close_socket(sock);
         JK_TRACE_EXIT(l);
-        return -1;
+        return JK_INVALID_SOCKET;
     }
 #endif
 #ifdef SO_LINGER
@@ -418,7 +424,7 @@ int jk_open_socket(struct sockaddr_in *addr, int keepalive,
                 "failed setting SO_LINGER with errno=%d", errno);
         jk_close_socket(sock);
         JK_TRACE_EXIT(l);
-        return -1;
+        return JK_INVALID_SOCKET;
     }
 #endif
     /* Tries to connect to Tomcat (continues trying while error is EINTR) */
@@ -445,7 +451,7 @@ iSeries when Unix98 is required at compil time */
                "connect to %s failed with errno=%d",
                jk_dump_hinfo(addr, buf), errno);
         jk_close_socket(sock);
-        sock = -1;
+        sock = JK_INVALID_SOCKET;
     }
     else {
         if (JK_IS_DEBUG_LEVEL(l))
@@ -458,7 +464,7 @@ iSeries when Unix98 is required at compil time */
 
 /** close the socket */
 
-int jk_close_socket(int s)
+int jk_close_socket(jk_sock_t s)
 {
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
     if (s != INVALID_SOCKET)
@@ -483,11 +489,12 @@ int jk_close_socket(int s)
 #define SHUT_WR 0x01
 #endif
 #endif
-int jk_shutdown_socket(int s)
+int jk_shutdown_socket(jk_sock_t s)
 {
     unsigned char dummy[512];
     int nbytes;
     int ttl = 0;
+    int rc = 0;
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
     int tmout = SECONDS_TO_LINGER * 1000;
     if (s == INVALID_SOCKET)
@@ -504,19 +511,21 @@ int jk_shutdown_socket(int s)
         return jk_close_socket(s);
     }
 #if defined(WIN32)  || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-    setsockopt(s, SOL_SOCKET, SO_RCVTIMEO,
-               (const char *) &tmout, sizeof(int));
+    if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO,
+                   (const char *) &tmout, sizeof(int)) == 0)
+        rc = 1;
 #elif defined(SO_RCVTIMEO) && defined(USE_SO_RCVTIMEO)
     tv.tv_sec  = SECONDS_TO_LINGER;
     tv.tv_usec = 0;
-    setsockopt(s, SOL_SOCKET, SO_RCVTIMEO,
-               (const void *) &tv, sizeof(tv));
+    if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO,
+                   (const void *) &tv, sizeof(tv)))
+        rc = 1;
 #endif
     /* Read all data from the peer until we reach "end-of-file" (FIN
      * from peer) or we've exceeded our overall timeout. If the client does
-     * not send us bytes within12 second, close the connection.
+     * not send us bytes within 12 second, close the connection.
      */
-    while (1) {
+    while (rc) {
         nbytes = jk_tcp_socket_recvfull(s, dummy, sizeof(dummy));
         if (nbytes <= 0)
             break;
@@ -537,7 +546,7 @@ int jk_shutdown_socket(int s)
  * @bug       this fails on Unixes if len is too big for the underlying
  *             protocol.
  */
-int jk_tcp_socket_sendfull(int sd, const unsigned char *b, int len)
+int jk_tcp_socket_sendfull(jk_sock_t sd, const unsigned char *b, int len)
 {
     int sent = 0;
     int wr;
@@ -571,7 +580,7 @@ int jk_tcp_socket_sendfull(int sd, const unsigned char *b, int len)
  * @return    <0: receive failed or connection closed.
  *            >0: length of the received data.
  */
-int jk_tcp_socket_recvfull(int sd, unsigned char *b, int len)
+int jk_tcp_socket_recvfull(jk_sock_t sd, unsigned char *b, int len)
 {
     int rdlen = 0;
     int rd;
@@ -615,56 +624,48 @@ char *jk_dump_hinfo(struct sockaddr_in *saddr, char *buf)
     return buf;
 }
 
-#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-int jk_is_socket_connected(int sd)
+int jk_is_socket_connected(jk_sock_t sock)
 {
     fd_set fd;
     struct timeval tv;
     int rc;
 
     FD_ZERO(&fd);
-    FD_SET(sd, &fd);
+    FD_SET(sock, &fd);
 
     /* Wait one microsecond */
     tv.tv_sec  = 0;
     tv.tv_usec = 1;
-
-    /* If we get a timeout, then we are still connected */
-    if ((rc = select(1, &fd, NULL, NULL, &tv)) == 0) {
-        errno = 0;
-        return 1;
-    }
-    else {
-        if (rc == SOCKET_ERROR)
-            errno = WSAGetLastError() - WSABASEERR;
-        else
-            errno = EOF;
-        return 0;
-    }
-}
-#else
-int jk_is_socket_connected(int sd)
-{
-    char test_buffer[1];
-    int  rd;
-    int  saved_errno;
-
-    errno = 0;
-    /* Set socket to nonblocking */
-    if (sononblock(sd) != 0)
-        return 0;
+    
     do {
-        rd = read(sd, test_buffer, 1);
-    } while (rd == -1 && errno == EINTR);
-    saved_errno = errno;
-    soblock(sd);
-    if (rd == -1 && saved_errno == EWOULDBLOCK) {
-    errno = 0;
+        rc = select((int)sock + 1, &fd, NULL, NULL, &tv);
+#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
+        errno = WSAGetLastError() - WSABASEERR;
+#endif        
+    } while (rc == -1 && errno == EINTR);
+
+    if (rc == 0) {
+        /* If we get a timeout, then we are still connected */
         return 1;
     }
-    else {
-        errno = saved_errno ? saved_errno : EOF;
-        return 0;
+    else if (rc == 1) {
+#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
+        u_long nr;
+        if (ioctlsocket(sock, FIONREAD, &nr) == 0) {
+            if (WSAGetLastError() == 0)
+                errno = 0;
+            else
+                errno = WSAGetLastError() - WSABASEERR;
+            return nr == 0 ? 0 : 1;
+        }
+        errno = WSAGetLastError() - WSABASEERR;
+#else
+        int nr;
+        if (ioctl(sock, FIONREAD, (void*)&nr) == 0) {
+            return nr == 0 ? 0 : 1;
+        }
+#endif        
     }
+
+    return 0;
 }
-#endif
