@@ -18,7 +18,7 @@
  * Description: In process JNI worker                                      *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Based on:                                                               *
- * Version:     $Revision: 300224 $                                           *
+ * Version:     $Revision: 434177 $                                           *
  ***************************************************************************/
 
 #if !defined(WIN32) && !defined(NETWARE) && !defined(AS400)
@@ -254,7 +254,10 @@ static int JK_METHOD service(jk_endpoint_t *e,
     jint rc;
 
     JK_TRACE_ENTER(l);
-    if (!e || !e->endpoint_private || !s) {
+
+    if (is_recoverable_error)
+        *is_recoverable_error = JK_FALSE;
+    if (!e || !e->endpoint_private || !s || !is_recoverable_error) {
         JK_LOG_NULL_PARAMS(l);
         JK_TRACE_EXIT(l);
         return JK_FALSE;
@@ -262,11 +265,6 @@ static int JK_METHOD service(jk_endpoint_t *e,
 
     p = e->endpoint_private;
 
-    if (!is_recoverable_error) {
-        JK_TRACE_EXIT(l);
-        return JK_FALSE;
-    }
-
     if (!p->attached) {
         /* Try to attach */
         if (!(p->env = attach_to_jvm(p->worker, l))) {
@@ -285,8 +283,6 @@ static int JK_METHOD service(jk_endpoint_t *e,
      * When we call the JVM we cannot know what happens
      * So we can not recover !!!
      */
-    *is_recoverable_error = JK_FALSE;
-
     jk_log(l, JK_LOG_DEBUG, "In service, calling Tomcat...");
 
     rc = (*(p->env))->CallIntMethod(p->env,
@@ -834,7 +830,7 @@ static int open_jvm1(jni_worker_t * p, JNIEnv ** env, jk_logger_t *l)
     jk_log(l, JK_LOG_DEBUG, "got default jvm args");
 
     if (vm_args.classpath) {
-        unsigned len = strlen(vm_args.classpath) +
+        size_t len = strlen(vm_args.classpath) +
             strlen(p->tomcat_classpath) + 3;
         char *tmp = jk_pool_alloc(&p->p, len);
         if (tmp) {
@@ -907,7 +903,7 @@ static int detect_jvm_version(jk_logger_t *l)
 static char *build_opt_str(jk_pool_t *p,
                            char *opt_name, char *opt_value, jk_logger_t *l)
 {
-    unsigned len = strlen(opt_name) + strlen(opt_value) + 2;
+    size_t len = strlen(opt_name) + strlen(opt_value) + 2;
 
     /* [V] IMHO, these should not be deallocated as long as the JVM runs */
     char *tmp = jk_pool_alloc(p, len);
@@ -927,7 +923,7 @@ static char *build_opt_int(jk_pool_t *p,
                            char *opt_name, int opt_value, jk_logger_t *l)
 {
     /* [V] this should suffice even for 64-bit int */
-    unsigned len = strlen(opt_name) + 20 + 2;
+    size_t len = strlen(opt_name) + 20 + 2;
     /* [V] IMHO, these should not be deallocated as long as the JVM runs */
     char *tmp = jk_pool_alloc(p, len);
 
