@@ -18,21 +18,22 @@
  * Description: General purpose map object                                 *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 300542 $                                          *
+ * Version:     $Revision: 439824 $                                          *
  ***************************************************************************/
 #ifdef AS400
 #include "apr_xlate.h"
 #endif
 
 #include "jk_global.h"
-#include "jk_map.h"
 #include "jk_pool.h"
-#include "jk_map.h"
 #include "jk_util.h"
-#include "jk_shm.h"
+#include "jk_map.h"
 
-#define CAPACITY_INC_SIZE (50)
-#define LENGTH_OF_LINE    (8192)
+#define CAPACITY_INC_SIZE   (50)
+#define LENGTH_OF_LINE      (8192)
+#define JK_MAP_RECURSION    (20)
+#define JK_MAP_REFERENCE    (".reference")
+#define JK_MAP_REFERENCE_SZ (strlen(JK_MAP_REFERENCE))
 
 #ifdef AS400
 #define CASE_MASK 0xbfbfbfbf
@@ -146,7 +147,7 @@ void *jk_map_get(jk_map_t *m, const char *name, const void *def)
         unsigned int key;
         COMPUTE_KEY_CHECKSUM(name, key)
         for (i = 0; i < m->size; i++) {
-            if (m->keys[i] == key && strcasecmp(m->names[i], name) == 0) {
+            if (m->keys[i] == key && strcmp(m->names[i], name) == 0) {
                 rc = m->values[i];
                 break;
             }
@@ -164,7 +165,7 @@ int jk_map_get_id(jk_map_t *m, const char *name)
         unsigned int key;
         COMPUTE_KEY_CHECKSUM(name, key)
         for (i = 0; i < m->size; i++) {
-            if (m->keys[i] == key && strcasecmp(m->names[i], name) == 0) {
+            if (m->keys[i] == key && strcmp(m->names[i], name) == 0) {
                 rc = i;
                 break;
             }
@@ -183,7 +184,7 @@ const char *jk_map_get_string(jk_map_t *m, const char *name, const char *def)
         unsigned int key;
         COMPUTE_KEY_CHECKSUM(name, key)
         for (i = 0; i < m->size; i++) {
-            if (m->keys[i] == key && strcasecmp(m->names[i], name) == 0) {
+            if (m->keys[i] == key && strcmp(m->names[i], name) == 0) {
                 rc = m->values[i];
                 break;
             }
@@ -311,6 +312,27 @@ char **jk_map_get_string_list(jk_map_t *m,
     return ar;
 }
 
+int jk_map_add(jk_map_t *m, const char *name, const void *value)
+{
+    int rc = JK_FALSE;
+
+    if (m && name) {
+        unsigned int key;
+        COMPUTE_KEY_CHECKSUM(name, key)
+        map_realloc(m);
+
+        if (m->size < m->capacity) {
+            m->values[m->size] = value;
+            m->names[m->size] = jk_pool_strdup(&m->p, name);
+            m->keys[m->size] = key;
+            m->size++;
+            rc = JK_TRUE;
+        }
+    }
+
+    return rc;
+}
+
 int jk_map_put(jk_map_t *m, const char *name, const void *value, void **old)
 {
     int rc = JK_FALSE;
@@ -332,22 +354,14 @@ int jk_map_put(jk_map_t *m, const char *name, const void *value, void **old)
             rc = JK_TRUE;
         }
         else {
-            map_realloc(m);
-
-            if (m->size < m->capacity) {
-                m->values[m->size] = value;
-                m->names[m->size] = jk_pool_strdup(&m->p, name);
-                m->keys[m->size] = key;
-                m->size++;
-                rc = JK_TRUE;
-            }
+            rc = jk_map_add(m, name, value);
         }
     }
 
     return rc;
 }
 
-int jk_map_read_property(jk_map_t *m, const char *str)
+int jk_map_read_property(jk_map_t *m, const char *str, jk_logger_t *l)
 {
     int rc = JK_TRUE;
     char buf[LENGTH_OF_LINE + 1];
@@ -366,23 +380,29 @@ int jk_map_read_property(jk_map_t *m, const char *str)
             trim(v);
             if (strlen(v) && strlen(prp)) {
                 const char *oldv = jk_map_get_string(m, prp, NULL);
-                v = jk_map_replace_properties(v, m);
+                v = jk_map_replace_properties(m, v);
                 if (oldv && jk_is_unique_property(prp) == JK_FALSE) {
                     char *tmpv = jk_pool_alloc(&m->p,
                                        strlen(v) + strlen(oldv) + 3);
                     if (tmpv) {
                         char sep = '*';
-                        if (jk_is_path_poperty(prp))
+                        if (jk_is_path_property(prp))
                             sep = PATH_SEPERATOR;
-                        else if (jk_is_cmd_line_poperty(prp))
+                        else if (jk_is_cmd_line_property(prp))
                             sep = ' ';
-                        else if (!strcasecmp(prp, "worker.list"))
+                        else if (jk_is_list_property(prp))
                             sep = ',';
                         sprintf(tmpv, "%s%c%s", oldv, sep, v);
                     }
                     v = tmpv;
                 }
                 else {
+                    if (jk_is_deprecated_property(prp)) {
+                        jk_log(l, JK_LOG_WARNING,
+                               "The attribute %s is deprecated - please check"
+                               " the documentation for the correct replacement.",
+                               prp);
+                    }
                     v = jk_pool_strdup(&m->p, v);
                 }
                 if (v) {
@@ -398,7 +418,7 @@ int jk_map_read_property(jk_map_t *m, const char *str)
 }
 
 
-int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified)
+int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified, jk_logger_t *l)
 {
     int rc = JK_FALSE;
 
@@ -422,7 +442,7 @@ int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified)
             while (NULL != (prp = fgets(buf, LENGTH_OF_LINE, fp))) {
                 trim_prp_comment(prp);
                 if (*prp) {
-                    if ((rc =jk_map_read_property(m, prp)) == JK_FALSE)
+                    if ((rc = jk_map_read_property(m, prp, l)) == JK_FALSE)
                         break;
                 }
             }
@@ -537,7 +557,7 @@ static int map_realloc(jk_map_t *m)
  *  Replace $(property) in value.
  *
  */
-char *jk_map_replace_properties(const char *value, jk_map_t *m)
+char *jk_map_replace_properties(jk_map_t *m, const char *value)
 {
     char *rc = (char *)value;
     char *env_start = rc;
@@ -598,3 +618,102 @@ char *jk_map_replace_properties(const char *value, jk_map_t *m)
 
     return rc;
 }
+
+/**
+ *  Resolve references
+ *
+ */
+int jk_map_resolve_references(jk_map_t *m, const char *prefix,
+                              int wildcard, int depth, jk_logger_t *l)
+{
+    int rc = JK_FALSE;
+
+    JK_TRACE_ENTER(l);
+
+    if (m && prefix && depth <= JK_MAP_RECURSION) {
+        size_t prelen = strlen(prefix);
+        unsigned int i;
+        rc = JK_TRUE;
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG,
+                   "Checking for references with prefix %s with%s wildcard (recursion %d)",
+                   prefix, wildcard? "" : "out", depth);
+        for (i = 0; i < m->size; i++) {
+            if (m->values[i] && !strncmp(m->names[i], prefix, prelen)) {
+                size_t remain = strlen(m->names[i]) - prelen;
+                if ((remain == JK_MAP_REFERENCE_SZ ) || (wildcard && remain > JK_MAP_REFERENCE_SZ)) {
+                    remain = strlen(m->names[i]) - JK_MAP_REFERENCE_SZ;
+                    if (!strncmp(m->names[i] + remain, JK_MAP_REFERENCE, JK_MAP_REFERENCE_SZ)) {
+                        char *from = jk_pool_alloc(&m->p,
+                                                   (sizeof(char) *
+                                                   (strlen(m->values[i]) + 2)));
+                        char *to = jk_pool_alloc(&m->p,
+                                                 (sizeof(char) *
+                                                 (remain + 2)));
+                        if (!from || !to) {
+                            rc = JK_FALSE;
+                            break;
+                        }
+                        strcpy(from, m->values[i]);
+                        *(from+strlen(m->values[i]))   = '.';
+                        *(from+strlen(m->values[i])+1) = '\0';
+                        strncpy(to, m->names[i], remain);
+                        *(to+remain)   = '.';
+                        *(to+remain+1) = '\0';
+
+                        rc = jk_map_resolve_references(m, m->values[i], 0, ++depth, l);
+                        if (rc == JK_FALSE) {
+                            break;
+                        }
+                        if (JK_IS_DEBUG_LEVEL(l))
+                            jk_log(l, JK_LOG_DEBUG,
+                                   "Copying values from %s to %s",
+                                   from, to);
+                        rc = jk_map_inherit_properties(m, from, to);
+                        if (rc == JK_FALSE) {
+                            break;
+                        }
+                        m->values[i] = NULL;
+                    }
+                }
+            }
+        }
+    }
+    JK_TRACE_EXIT(l);
+    return rc;
+}
+
+/**
+ *  Inherit properties
+ *
+ */
+int jk_map_inherit_properties(jk_map_t *m, const char *from, const char *to)
+{
+    int rc = JK_FALSE;
+
+    if (m && from && to) {
+        unsigned int i;
+        rc = JK_TRUE;
+        for (i = 0; i < m->size; i++) {
+            if (!strncmp(m->names[i], from, strlen(from))) {
+                const char *prp = m->names[i] + strlen(from);
+                char *to_prp = jk_pool_alloc(&m->p,
+                                             (sizeof(char) *
+                                             (strlen(to) +
+                                             strlen(prp) + 1)));
+                if (!to_prp) {
+                    break;
+                }
+                strcpy(to_prp, to);
+                strcat(to_prp, prp);
+                if (jk_map_get_id(m, to_prp) < 0 ) {
+                    rc = jk_map_add(m, to_prp, m->values[i]);
+                    if (rc == JK_FALSE) {
+                        break;
+                    }
+                }
+            }
+        }
+    }
+    return rc;
+}
