@@ -21,7 +21,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Dan Milstein <danmil@shore.net>                            *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 300634 $                                          *
+ * Version:     $Revision: 434345 $                                          *
  ***************************************************************************/
 
 #ifndef JK_SERVICE_H
@@ -34,7 +34,7 @@
 #include "jk_uri_worker_map.h"
 #include "jk_msg_buff.h"
 
-#define JK_RETRIES 3
+#define JK_RETRIES 2
 
 #ifdef __cplusplus
 extern "C"
@@ -200,11 +200,6 @@ struct jk_ws_service
        and it'll work.
      */
     const char *secret;
-    /*
-     * Callbacks into the web server.  For each, the first argument is
-     * essentially a 'this' pointer.  All return JK_TRUE on success
-     * and JK_FALSE on failure.
-     */
 
     /*
      * Area to get POST data for fail-over recovery in POST
@@ -225,6 +220,11 @@ struct jk_ws_service
      */
     jk_uri_worker_map_t *uw_map;
 
+    /*
+     * Callbacks into the web server.  For each, the first argument is
+     * essentially a 'this' pointer.  All return JK_TRUE on success
+     * and JK_FALSE on failure.
+     */
     /*
      * Send the response headers to the browser.
      */
@@ -254,6 +254,19 @@ struct jk_ws_service
      * Flush a chunk of response data back to the browser.
      */
     void (JK_METHOD * flush) (jk_ws_service_t *s);
+
+    /*
+     * If set do not reuse socket after each full response
+     */
+    int disable_reuse;
+
+    /*
+     * Add more data to log facilities.
+     */
+    void (JK_METHOD * add_log_items) (jk_ws_service_t *s,
+                                      const char *const *log_names,
+                                      const char *const *log_values,
+                                      unsigned num_of_items);
 };
 
 /*
@@ -425,7 +438,7 @@ struct jk_worker
     /*
      * Maintain this worker.
      */
-    int (JK_METHOD * maintain) (jk_worker_t *w, jk_logger_t *l);
+    int (JK_METHOD * maintain) (jk_worker_t *w, time_t now, jk_logger_t *l);
 
 };
 
