@@ -17,7 +17,7 @@
 /***************************************************************************
  * Description: Socket buffer header file                                  *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 299829 $                                           *
+ * Version:     $Revision: 424266 $                                           *
  ***************************************************************************/
 
 #include "jk_global.h"
@@ -27,13 +27,13 @@
 struct jk_sockbuf
 {
     char buf[SOCKBUF_SIZE];
-    unsigned start;
-    unsigned end;
-    int sd;
+    unsigned int start;
+    unsigned int end;
+    jk_sock_t sd;
 };
 typedef struct jk_sockbuf jk_sockbuf_t;
 
-int jk_sb_open(jk_sockbuf_t *sb, int sd);
+int jk_sb_open(jk_sockbuf_t *sb, jk_sock_t sd);
 
 int jk_sb_write(jk_sockbuf_t *sb, const void *buf, unsigned sz);
 
