@@ -17,7 +17,8 @@
 /***************************************************************************
  * Description: Status worker, display and manages JK workers              *
  * Author:      Mladen Turk <mturk@jboss.com>                              *
- * Version:     $Revision: 386279 $                                           *
+ * Author:      Rainer Jung <rjung@apache.org>                             *
+ * Version:     $Revision: 441427 $                                           *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -44,7 +45,7 @@
 #define JK_STATUS_BEND "</body>\n</html>\n"
 
 #define JK_STATUS_XMLH "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"          \
-                       "<jk:status xmlns:jk=\"http://jakarta.apache.org\">\n"
+                       "<jk:status xmlns:jk=\"http://tomcat.apache.org\">\n"
 
 #define JK_STATUS_XMLE "</jk:status>\n"
 
@@ -90,14 +91,6 @@ static const char *headers_names[] = {
     NULL
 };
 
-static const char *lb_method_type[] = {
-    JK_LB_METHOD_REQUESTS,
-    JK_LB_METHOD_TRAFFIC,
-    JK_LB_METHOD_BUSYNESS,
-    "unknown",
-    NULL
-};
-
 #define HEADERS_NO_CACHE "no-cache", "no-cache", NULL
 
 static const char *headers_vhtml[] = {
@@ -174,7 +167,7 @@ static int jk_printf(jk_ws_service_t *s, const char *fmt, ...)
 }
 
 /* Actually APR's apr_strfsize */
-static char *status_strfsize(jk_u64_t size, char *buf)
+static char *status_strfsize(jk_uint64_t size, char *buf)
 {
     const char ord[] = "KMGTPE";
     const char *o = ord;
@@ -224,22 +217,6 @@ static const char *status_val_bool(int v)
         return "True";
 }
 
-static const char *status_val_status(int s, int d, int e, int r, int b)
-{
-    if (s)
-        return "Stopped";
-    else if (d)
-        return "Disabled";
-    else if (r)
-        return "Recovering";
-    else if (e)
-        return "Error";
-    else if (b)
-        return "Busy";
-    else
-        return "OK";
-}
-
 static const char *status_val_match(unsigned int match)
 {
     if (match & MATCH_TYPE_STOPPED)
@@ -267,7 +244,7 @@ static const char *status_val_match(unsigned int match)
 static void jk_puts(jk_ws_service_t *s, const char *str)
 {
     if (str)
-        s->write(s, str, strlen(str));
+        s->write(s, str, (unsigned int)strlen(str));
     else
         s->write(s, "(null)", 6);
 }
@@ -282,7 +259,7 @@ static void jk_putv(jk_ws_service_t *s, ...)
         str = va_arg(va, const char *);
         if (str == NULL)
             break;
-        s->write(s, str, strlen(str));
+        s->write(s, str, (unsigned int)strlen(str));
     }
     va_end(va);
 }
@@ -296,8 +273,15 @@ static const char *status_cmd(const char *param, const char *req, char *buf, siz
     *buf = '\0';
     if (!req)
         return NULL;
+    if (!param)
+        return NULL;
     sprintf(ps, "&%s=", param);
     p = strstr(req, ps);
+    if (!p) {
+        sprintf(ps, "%s=", param);
+        if (!strncmp(req, ps, strlen(ps)))
+            p = (char *)req;
+    }
     if (p) {
         p += strlen(ps);
         while (*p) {
@@ -408,6 +392,7 @@ static void display_workers(jk_ws_service_t *s, status_worker_t *sw,
 {
     unsigned int i;
     char buf[32];
+    time_t now = time(NULL);
 
     for (i = 0; i < sw->we->num_of_workers; i++) {
         jk_worker_t *w = wc_get_worker_for_name(sw->we->worker_list[i], l);
@@ -444,6 +429,7 @@ static void display_workers(jk_ws_service_t *s, status_worker_t *sw,
                     "<th>Retries</th>"
                     "<th>Method</th>"
                     "<th>Lock</th>"
+                    "<th>Recovery timeout</th>"
                     "</tr>\n<tr>");
             jk_putv(s, "<td>", status_worker_type(w->type), "</td>", NULL);
             jk_putv(s, "<td>", status_val_bool(lb->s->sticky_session),
@@ -451,13 +437,14 @@ static void display_workers(jk_ws_service_t *s, status_worker_t *sw,
             jk_putv(s, "<td>", status_val_bool(lb->s->sticky_session_force),
                     "</td>", NULL);
             jk_printf(s, "<td>%d</td>", lb->s->retries);
-            jk_printf(s, "<td>%s</td>", lb_method_type[lb->lbmethod]);
-            jk_printf(s, "<td>%s</td>", lb->lblock == JK_LB_LOCK_DEFAULT ? JK_LB_LM_DEFAULT : JK_LB_LM_PESSIMISTIC);
+            jk_printf(s, "<td>%s</td>", jk_lb_get_method(lb, l));
+            jk_printf(s, "<td>%s</td>", jk_lb_get_lock(lb, l));
+            jk_printf(s, "<td>%d</td>", lb->s->recover_wait_time);
             jk_puts(s, "</tr>\n</table>\n<br/>\n");
             jk_puts(s, "<table><tr>"
-                    "<th>Name</th><th>Type</th><th>Host</th><th>Addr</th>"
-                    "<th>Stat</th><th>F</th><th>V</th><th>Acc</th><th>Err</th>"
-                    "<th>Wr</th><th>Rd</th><th>Busy</th><th>Max</th><th>RR</th><th>Cd</th></tr>\n");
+                    "<th>Name</th><th>Type</th><th>jvmRoute</th><th>Host</th><th>Addr</th>"
+                    "<th>Act</th><th>Stat</th><th>D</th><th>F</th><th>M</th><th>V</th><th>Acc</th><th>Err</th><th>CE</th>"
+                    "<th>Wr</th><th>Rd</th><th>Busy</th><th>Max</th><th>RR</th><th>Cd</th><th>Rs</th></tr>\n");
             for (j = 0; j < lb->num_of_workers; j++) {
                 worker_record_t *wr = &(lb->lb_workers[j]);
                 ajp_worker_t *a = (ajp_worker_t *)wr->w->worker_private;
@@ -468,71 +455,93 @@ static void display_workers(jk_ws_service_t *s, status_worker_t *sw,
                 if (dworker && strcmp(dworker, wr->s->name) == 0)
                     selected = j;
                 jk_putv(s, "<td>", status_worker_type(wr->w->type), "</td>", NULL);
+                jk_putv(s, "<td>", wr->s->jvm_route, "</td>", NULL);
                 jk_printf(s, "<td>%s:%d</td>", a->host, a->port);
                 jk_putv(s, "<td>", jk_dump_hinfo(&a->worker_inet_addr, buf),
                         "</td>", NULL);
                 /* TODO: descriptive status */
-                jk_putv(s, "<td>",
-                        status_val_status(wr->s->is_stopped,
-                                          wr->s->is_disabled,
-                                          wr->s->in_error_state,
-                                          wr->s->in_recovering,
-                                          wr->s->is_busy),
-                        "</td>", NULL);
+                jk_putv(s, "<td>", jk_lb_get_activation(wr, l), "</td>", NULL);
+                jk_putv(s, "<td>", jk_lb_get_state(wr, l), "</td>", NULL);
+                jk_printf(s, "<td>%d</td>", wr->s->distance);
                 jk_printf(s, "<td>%d</td>", wr->s->lb_factor);
-                jk_printf(s, "<td>%d</td>", wr->s->lb_value);
-                jk_printf(s, "<td>%u</td>", wr->s->elected);
-                jk_printf(s, "<td>%u</td>", wr->s->errors);
+                jk_printf(s, "<td>%" JK_UINT64_T_FMT "</td>", wr->s->lb_mult);
+                jk_printf(s, "<td>%" JK_UINT64_T_FMT "</td>", wr->s->lb_value);
+                jk_printf(s, "<td>%" JK_UINT64_T_FMT "</td>", wr->s->elected);
+                jk_printf(s, "<td>%" JK_UINT32_T_FMT "</td>", wr->s->errors);
+                jk_printf(s, "<td>%" JK_UINT32_T_FMT "</td>", wr->s->client_errors);
                 jk_putv(s, "<td>", status_strfsize(wr->s->transferred, buf),
                         "</td>", NULL);
                 jk_putv(s, "<td>", status_strfsize(wr->s->readed, buf),
                         "</td>", NULL);
                 jk_printf(s, "<td>%u</td>", wr->s->busy);
-                jk_printf(s, "<td>%u</td><td>", wr->s->max_busy);
+                jk_printf(s, "<td>%u</td>\n<td>", wr->s->max_busy);
                 if (wr->s->redirect && *wr->s->redirect)
                     jk_puts(s, wr->s->redirect);
                 else
                     jk_puts(s,"&nbsp;");
-                jk_puts(s, "</td><td>\n");
+                jk_puts(s, "</td>\n<td>");
                 if (wr->s->domain && *wr->s->domain)
                     jk_puts(s, wr->s->domain);
                 else
                     jk_puts(s,"&nbsp;");
+                jk_puts(s, "</td>\n<td>");
+                if (wr->s->state == JK_LB_STATE_ERROR) {
+                    int rs = lb->maintain_time - (int)difftime(now, lb->s->last_maintain_time);
+                    if (rs < lb->s->recover_wait_time - (int)difftime(now, wr->s->error_time))
+                        rs += lb->maintain_time;
+                    jk_printf(s, "%u", rs < 0 ? 0 : rs);
+                }
+                else
+                    jk_puts(s, "-");
                 jk_puts(s, "</td>\n</tr>\n");
             }
             jk_puts(s, "</table><br/>\n");
             if (selected >= 0) {
                 worker_record_t *wr = &(lb->lb_workers[selected]);
                 jk_putv(s, "<hr/><h3>Edit worker settings for ",
-                        wr->s->name, "</h3>\n", NULL);
+                        wr->s->name, NULL);
                 jk_putv(s, "<form method=\"GET\" action=\"",
                         s->req_uri, "\">\n", NULL);
-                jk_puts(s, "<table>\n<input type=\"hidden\" name=\"cmd\" ");
+                jk_puts(s, "<input type=\"hidden\" name=\"cmd\" ");
                 jk_puts(s, "value=\"update\">\n");
                 jk_puts(s, "<input type=\"hidden\" name=\"w\" ");
                 jk_putv(s, "value=\"", wr->s->name, "\">\n", NULL);
                 jk_puts(s, "<input type=\"hidden\" name=\"id\" ");
-                jk_printf(s, "value=\"%u\">\n</table>\n", selected);
+                jk_printf(s, "value=\"%u\">\n", selected);
                 jk_puts(s, "<input type=\"hidden\" name=\"lb\" ");
-                jk_printf(s, "value=\"%u\">\n</table>\n", i);
+                jk_printf(s, "value=\"%u\">\n", i);
 
-                jk_puts(s, "<table>\n<tr><td>Load factor:</td><td><input name=\"wf\" type=\"text\" ");
-                jk_printf(s, "value=\"%d\"/></td><tr>\n", wr->s->lb_factor);
+                jk_puts(s, "<table>\n<tr><td>Route:</td><td><input name=\"wn\" type=\"text\" ");
+                jk_printf(s, "value=\"%s\"/></td></tr>\n", wr->s->jvm_route);
+
+                jk_puts(s, "<tr><td>Distance:</td><td><input name=\"wx\" type=\"text\" ");
+                jk_printf(s, "value=\"%d\"/></td></tr>\n", wr->s->distance);
+
+                jk_puts(s, "<tr><td>Load factor:</td><td><input name=\"wf\" type=\"text\" ");
+                jk_printf(s, "value=\"%d\"/></td></tr>\n", wr->s->lb_factor);
                 jk_puts(s, "<tr><td>Route Redirect:</td><td><input name=\"wr\" type=\"text\" ");
                 jk_putv(s, "value=\"", wr->s->redirect, NULL);
                 jk_puts(s, "\"/></td></tr>\n");
                 jk_puts(s, "<tr><td>Cluster Domain:</td><td><input name=\"wc\" type=\"text\" ");
                 jk_putv(s, "value=\"", wr->s->domain, NULL);
                 jk_puts(s, "\"/></td></tr>\n");
-                jk_puts(s, "<tr><td>Disabled:</td><td><input name=\"wd\" type=\"checkbox\"");
-                if (wr->s->is_disabled)
-                    jk_puts(s, "  checked=\"checked\"");
+                jk_puts(s, "<tr><td>Activation:</td><td></td></tr>\n");
+                jk_puts(s, "<tr><td>&nbsp;&nbsp;Active</td><td><input name=\"wa\" type=\"radio\"");
+                jk_printf(s, " value=\"%d\"", JK_LB_ACTIVATION_ACTIVE);
+                if (wr->s->activation == JK_LB_ACTIVATION_ACTIVE)
+                    jk_puts(s, " checked=\"checked\"");
                 jk_puts(s, "/></td></tr>\n");
-                jk_puts(s, "<tr><td>Stopped:</td><td><input name=\"ws\" type=\"checkbox\"");
-                if (wr->s->is_stopped)
-                    jk_puts(s, "  checked=\"checked\"");
+                jk_puts(s, "<tr><td>&nbsp;&nbsp;Disabled</td><td><input name=\"wa\" type=\"radio\"");
+                jk_printf(s, " value=\"%d\"", JK_LB_ACTIVATION_DISABLED);
+                if (wr->s->activation == JK_LB_ACTIVATION_DISABLED)
+                    jk_puts(s, " checked=\"checked\"");
                 jk_puts(s, "/></td></tr>\n");
-                jk_puts(s, "</td></tr>\n</table>\n");
+                jk_puts(s, "<tr><td>&nbsp;&nbsp;Stopped</td><td><input name=\"wa\" type=\"radio\"");
+                jk_printf(s, " value=\"%d\"", JK_LB_ACTIVATION_STOPPED);
+                if (wr->s->activation == JK_LB_ACTIVATION_STOPPED)
+                    jk_puts(s, " checked=\"checked\"");
+                jk_puts(s, "/></td></tr>\n");
+                jk_puts(s, "</table>\n");
                 jk_puts(s, "<br/><input type=\"submit\" value=\"Update Worker\"/>\n</form>\n");
 
             }
@@ -542,12 +551,12 @@ static void display_workers(jk_ws_service_t *s, status_worker_t *sw,
                         dworker, "</h3>\n", NULL);
                 jk_putv(s, "<form method=\"GET\" action=\"",
                         s->req_uri, "\">\n", NULL);
-                jk_puts(s, "<table>\n<input type=\"hidden\" name=\"cmd\" ");
+                jk_puts(s, "<input type=\"hidden\" name=\"cmd\" ");
                 jk_puts(s, "value=\"update\"/>\n");
                 jk_puts(s, "<input type=\"hidden\" name=\"w\" ");
                 jk_putv(s, "value=\"", dworker, "\"/>\n", NULL);
                 jk_puts(s, "<input type=\"hidden\" name=\"id\" ");
-                jk_printf(s, "value=\"%u\"/>\n</table>\n", i);
+                jk_printf(s, "value=\"%u\"/>\n", i);
 
                 jk_puts(s, "<table>\n<tr><td>Retries:</td><td><input name=\"lr\" type=\"text\" ");
                 jk_printf(s, "value=\"%d\"/></td></tr>\n", lb->s->retries);
@@ -572,7 +581,6 @@ static void display_workers(jk_ws_service_t *s, status_worker_t *sw,
                     "<th>Type</th><th>Host</th><th>Addr</th>"
                     "</tr>\n<tr>");
             jk_putv(s, "<td>", status_worker_type(w->type), "</td>", NULL);
-            jk_puts(s, "</tr>\n</table>\n");
             jk_printf(s, "<td>%s:%d</td>", aw->host, aw->port);
             jk_putv(s, "<td>", jk_dump_hinfo(&aw->worker_inet_addr, buf),
                     "</td>\n</tr>\n", NULL);
@@ -582,20 +590,26 @@ static void display_workers(jk_ws_service_t *s, status_worker_t *sw,
     }
     /* Display legend */
     jk_puts(s, "<hr/><table>\n"
-            "<tr><th>Name</th><td>Worker route name</td></tr>\n"
+            "<tr><th>Name</th><td>Worker name</td></tr>\n"
             "<tr><th>Type</th><td>Worker type</td></tr>\n"
+            "<tr><th>jvmRoute</th><td>Worker JVM route</td></tr>\n"
             "<tr><th>Addr</th><td>Backend Address info</td></tr>\n"
-            "<tr><th>Stat</th><td>Worker status</td></tr>\n"
-            "<tr><th>F</th><td>Load Balancer Factor</td></tr>\n"
-            "<tr><th>V</th><td>Load Balancer Value</td></tr>\n"
+            "<tr><th>Act</th><td>Worker activation configuration</td></tr>\n"
+            "<tr><th>Stat</th><td>Worker error status</td></tr>\n"
+            "<tr><th>D</th><td>Worker distance</td></tr>\n"
+            "<tr><th>F</th><td>Load Balancer factor</td></tr>\n"
+            "<tr><th>M</th><td>Load Balancer multiplicity</td></tr>\n"
+            "<tr><th>V</th><td>Load Balancer value</td></tr>\n"
             "<tr><th>Acc</th><td>Number of requests</td></tr>\n"
             "<tr><th>Err</th><td>Number of failed requests</td></tr>\n"
+            "<tr><th>CE</th><td>Number of client errors</td></tr>\n"
             "<tr><th>Wr</th><td>Number of bytes transferred/min</td></tr>\n"
             "<tr><th>Rd</th><td>Number of bytes read/min</td></tr>\n"
             "<tr><th>Busy</th><td>Current number of busy connections</td></tr>\n"
             "<tr><th>Max</th><td>Maximum number of busy connections</td></tr>\n"
             "<tr><th>RR</th><td>Route redirect</td></tr>\n"
             "<tr><th>Cd</th><td>Cluster domain</td></tr>\n"
+            "<tr><th>Rs</th><td>Recovery scheduled</td></tr>\n"
             "</table>");
 }
 
@@ -646,26 +660,29 @@ static void dump_config(jk_ws_service_t *s, status_worker_t *sw,
             worker_record_t *wr = &(lb->lb_workers[j]);
             ajp_worker_t *a = (ajp_worker_t *)wr->w->worker_private;
             /* TODO: descriptive status */
-            jk_printf(s, "      <jk:member id=\"%d\" name=\"%s\" type=\"%s\" host=\"%s\" port=\"%d\" address=\"%s\" status=\"%s\"",
+            jk_printf(s, "      <jk:member id=\"%d\" name=\"%s\" type=\"%s\" host=\"%s\" port=\"%d\" address=\"%s\" activation=\"%s\" state=\"%s\"",
                 j,
                 wr->s->name,
                 status_worker_type(wr->w->type),
                 a->host,
                 a->port,
                 jk_dump_hinfo(&a->worker_inet_addr, buf),
-                status_val_status(wr->s->is_stopped,
-                                  wr->s->is_disabled,
-                                  wr->s->in_error_state,
-                                  wr->s->in_recovering,
-                                  wr->s->is_busy) );
+                jk_lb_get_activation(wr, l),
+                jk_lb_get_state(wr, l));
 
+            jk_printf(s, " distance=\"%d\"", wr->s->distance);
             jk_printf(s, " lbfactor=\"%d\"", wr->s->lb_factor);
-            jk_printf(s, " lbvalue=\"%d\"", wr->s->lb_value);
-            jk_printf(s, " elected=\"%u\"", wr->s->elected);
-            jk_printf(s, " readed=\"%u\"", wr->s->readed);
-            jk_printf(s, " transferred=\"%u\"", wr->s->transferred);
-            jk_printf(s, " errors=\"%u\"", wr->s->errors);
+            jk_printf(s, " lbmult=\"%" JK_UINT64_T_FMT "\"", wr->s->lb_mult);
+            jk_printf(s, " lbvalue=\"%" JK_UINT64_T_FMT "\"", wr->s->lb_value);
+            jk_printf(s, " elected=\"%" JK_UINT64_T_FMT "\"", wr->s->elected);
+            jk_printf(s, " errors=\"%" JK_UINT32_T_FMT "\"", wr->s->errors);
+            jk_printf(s, " clienterrors=\"%" JK_UINT32_T_FMT "\"", wr->s->client_errors);
+            jk_printf(s, " transferred=\"%" JK_UINT64_T_FMT "\"", wr->s->transferred);
+            jk_printf(s, " readed=\"%" JK_UINT64_T_FMT "\"", wr->s->readed);
             jk_printf(s, " busy=\"%u\"", wr->s->busy);
+            jk_printf(s, " maxbusy=\"%u\"", wr->s->max_busy);
+            if (wr->s->jvm_route && *wr->s->jvm_route)
+                jk_printf(s, " jvm_route=\"%s\"", wr->s->jvm_route);
             if (wr->s->redirect && *wr->s->redirect)
                 jk_printf(s, " redirect=\"%s\"", wr->s->redirect);
             if (wr->s->domain && *wr->s->domain)
@@ -696,8 +713,9 @@ static void update_worker(jk_ws_service_t *s, status_worker_t *sw,
         if (i > 0)
             lb->s->retries = i;
         i = status_int("lt", s->query_string, lb->s->recover_wait_time);
-        if (i > 59)
-            lb->s->recover_wait_time = i;
+        if (i < 1)
+            i = 1;
+        lb->s->recover_wait_time = i;
         lb->s->sticky_session = status_bool("ls", s->query_string);
         lb->s->sticky_session_force = status_bool("lf", s->query_string);
     }
@@ -738,11 +756,30 @@ static void update_worker(jk_ws_service_t *s, status_worker_t *sw,
             strncpy(wr->s->domain, b, JK_SHM_STR_SIZ);
         else
             memset(wr->s->domain, 0, JK_SHM_STR_SIZ);
-        wr->s->is_disabled = status_bool("wd", s->query_string);
-        wr->s->is_stopped = status_bool("ws", s->query_string);
+        if ((b = status_cmd("wn", s->query_string, buf, sizeof(buf)))) {
+            if (strlen(b) > 0)
+                strncpy(wr->s->jvm_route, b, JK_SHM_STR_SIZ);
+        }
+        i = status_int("wa", s->query_string, wr->s->activation);
+        if (wr->s->activation != i && i>0 && i<= JK_LB_ACTIVATION_STOPPED) {
+            wr->s->activation = i;
+            reset_lb_values(lb, l);
+            jk_log(l, JK_LOG_INFO,
+                   "worker '%s' activation changed to '%s' via status worker",
+                   wr->s->name,
+                   jk_lb_get_activation(wr, l));
+        }
+        i = status_int("wx", s->query_string, wr->s->distance);
+        if (wr->s->distance != i) {
+            wr->s->distance = i;
+            reset_lb_values(lb, l);
+        }
         i = status_int("wf", s->query_string, wr->s->lb_factor);
-        if (i > 0)
+        if (i > 0 && wr->s->lb_factor != i) {
             wr->s->lb_factor = i;
+            /* Recalculate the load multiplicators wrt. lb_factor */
+            update_mult(lb, l);
+        }
     }
 }
 
@@ -765,9 +802,7 @@ static void reset_worker(jk_ws_service_t *s, status_worker_t *sw,
             wr->s->max_busy         = 0;
             wr->s->readed           = 0;
             wr->s->transferred      = 0;
-            wr->s->is_busy          = JK_FALSE;
-            wr->s->in_error_state   = JK_FALSE;
-            wr->s->in_recovering    = JK_FALSE;
+            wr->s->state            = JK_LB_STATE_NA;
         }
     }
 }
@@ -807,90 +842,155 @@ static int JK_METHOD service(jk_endpoint_t *e,
                              jk_ws_service_t *s,
                              jk_logger_t *l, int *is_recoverable_error)
 {
+    char buf[128];
+    char *worker = NULL;
+    int cmd;
+    int mime;
+    status_endpoint_t *p;
+
     JK_TRACE_ENTER(l);
 
-    if (e && e->endpoint_private && s) {
-        char buf[128];
-        char *worker = NULL;
-        int cmd;
-        int mime;
-        status_endpoint_t *p = e->endpoint_private;
-
+    if (is_recoverable_error)
         *is_recoverable_error = JK_FALSE;
+    if (!e || !e->endpoint_private || !s || !is_recoverable_error) {
+        JK_LOG_NULL_PARAMS(l);
+        JK_TRACE_EXIT(l);
+        return JK_FALSE;
+    }
 
-        /* Step 1: Process GET params and update configuration */
-        cmd = status_cmd_type(s->query_string);
-        mime = status_mime_type(s->query_string);
-        if (cmd > 0 && (status_cmd("w", s->query_string, buf, sizeof(buf)) != NULL))
-            worker = strdup(buf);
-        if ((cmd == 2) && worker) {
-            /* lock shared memory */
-            jk_shm_lock();
-            update_worker(s, p->s_worker, worker, l);
-            /* update modification time to reflect the current config */
-            jk_shm_set_workers_time(time(NULL));
-            /* Since we updated the config no need to reload
-             * on the next request
-             */
-            jk_shm_sync_access_time();
-            /* unlock the shared memory */
-            jk_shm_unlock();
+    p = e->endpoint_private;
+
+    /* Step 1: Process GET params and update configuration */
+    cmd = status_cmd_type(s->query_string);
+    mime = status_mime_type(s->query_string);
+    if (cmd > 0 && (status_cmd("w", s->query_string, buf, sizeof(buf)) != NULL))
+        worker = strdup(buf);
+    if ((cmd == 2) && worker) {
+        /* lock shared memory */
+        jk_shm_lock();
+        update_worker(s, p->s_worker, worker, l);
+        /* update modification time to reflect the current config */
+        jk_shm_set_workers_time(time(NULL));
+        /* Since we updated the config no need to reload
+         * on the next request
+         */
+        jk_shm_sync_access_time();
+        /* unlock the shared memory */
+        jk_shm_unlock();
+    }
+    else if ((cmd == 3) && worker) {
+        /* lock shared memory */
+        jk_shm_lock();
+        reset_worker(s, p->s_worker, worker, l);
+        /* update modification time to reflect the current config */
+        jk_shm_set_workers_time(time(NULL));
+        /* Since we updated the config no need to reload
+         * on the next request
+         */
+        jk_shm_sync_access_time();
+        /* unlock the shared memory */
+        jk_shm_unlock();
+    }
+    if (mime == 0) {
+        int refresh = status_int("refresh", s->query_string, -1);
+        s->start_response(s, 200, "OK", headers_names, headers_vhtml, 3);
+        s->write(s, JK_STATUS_HEAD, sizeof(JK_STATUS_HEAD) - 1);
+        if (cmd > 1) {
+            jk_putv(s, "\n<meta http-equiv=\"Refresh\" content=\"0;url=",
+                      s->req_uri, "\">", NULL);
         }
-        else if ((cmd == 3) && worker) {
-            /* lock shared memory */
-            jk_shm_lock();
-            reset_worker(s, p->s_worker, worker, l);
-            /* update modification time to reflect the current config */
-            jk_shm_set_workers_time(time(NULL));
-            /* Since we updated the config no need to reload
-             * on the next request
-             */
-            jk_shm_sync_access_time();
-            /* unlock the shared memory */
-            jk_shm_unlock();
+        else if (cmd == 0 && refresh >= 0) {
+            jk_printf(s, "\n<meta http-equiv=\"Refresh\" content=\"%d;url=%s?%s\">",
+                      refresh, s->req_uri, s->query_string);
         }
-        if (mime == 0) {
-            s->start_response(s, 200, "OK", headers_names, headers_vhtml, 3);
-            s->write(s, JK_STATUS_HEAD, sizeof(JK_STATUS_HEAD) - 1);
-            if (p->s_worker->css) {
-                jk_putv(s, "\n<link rel=\"stylesheet\" type=\"text/css\" href=\"",
-                        p->s_worker->css, "\" />\n", NULL);
-            }
-            s->write(s, JK_STATUS_HEND, sizeof(JK_STATUS_HEND) - 1);
+        if (p->s_worker->css) {
+            jk_putv(s, "\n<link rel=\"stylesheet\" type=\"text/css\" href=\"",
+                    p->s_worker->css, "\" />\n", NULL);
+        }
+        s->write(s, JK_STATUS_HEND, sizeof(JK_STATUS_HEND) - 1);
+        jk_printf(s, "<!-- TOMCAT_CONNECTOR_VER_%02d%02d%02d -->\n", JK_VERMAJOR, JK_VERMINOR, JK_VERFIX);
+        jk_printf(s, "<!-- TOMCAT_CONNECTOR_STR_%s -->\n", JK_EXPOSED_VERSION);
+        if ( cmd <= 1 ) {
             jk_puts(s, "<h1>JK Status Manager for ");
             jk_puts(s, s->server_name);
             jk_puts(s, "</h1>\n\n");
-            jk_putv(s, "<dl><dt>Server Version: ",
-                    s->server_software, "</dt>\n", NULL);
-            jk_putv(s, "<dt>JK Version: ",
-                    JK_VERSTRING, "\n</dt></dl>\n", NULL);
+            jk_putv(s, "<dl><dt>Server Version:</dt><dd>",
+                    s->server_software, "</dd>\n", NULL);
+            jk_putv(s, "<dt>JK Version:</dt><dd>",
+                    JK_VERSTRING, "\n</dd></dl>\n", NULL);
+        }
+        if ( cmd == 0 ) {
+            if (refresh >= 0) {
+                const char *str = s->query_string;
+                char *buf = jk_pool_alloc(s->pool, sizeof(char *) * (strlen(str)+1));
+                int result = 0;
+                int scan = 0;
+
+                while (str[scan] != 0) {
+                    if (strncmp(&str[scan], "refresh=", strlen("refresh=")) == 0) {
+                        scan += strlen("refresh=");
+                        while (str[scan] != 0 && str[scan] != '&')
+                            scan++;
+                        if (str[scan] == '&')
+                            scan++;
+                    }
+                    else {
+                        if (result > 0 && str[scan] != 0 && str[scan] != '&') {
+                            buf[result] = '&';
+                            result++;
+                        }
+                        while (str[scan] != 0 && str[scan] != '&') {
+                            buf[result] = str[scan];
+                            result++;
+                            scan++;
+                        }
+                        if (str[scan] == '&')
+                            scan++;
+                    }
+                }
+                buf[result] = '\0';
+
+                jk_putv(s, "[<a href=\"", s->req_uri, NULL);
+                if (buf && buf[0])
+                    jk_putv(s, "?", buf, NULL);
+                jk_puts(s, "\">Stop auto refresh</a>]");
+            }
+            else {
+                jk_putv(s, "<form method=\"GET\" action=\"",
+                        s->req_uri, NULL);
+                if (s->query_string && s->query_string[0])
+                    jk_putv(s, "?", s->query_string, NULL);
+                jk_puts(s, "\">\n");
+                jk_puts(s, "<input type=\"submit\" value=\"Start auto refresh\"/>\n");
+                jk_puts(s, "(interval "
+                        "<input name=\"refresh\" type=\"text\" size=\"3\" value=\"10\"/> "
+                        "seconds)\n");
+                jk_puts(s, "</form>\n");
+            }
+        }
+        if ( cmd <= 1 ) {
             /* Step 2: Display configuration */
             display_workers(s, p->s_worker, worker, l);
-
-
-            s->write(s, JK_STATUS_BEND, sizeof(JK_STATUS_BEND) - 1);
-
         }
-        else if (mime == 1) {
-            s->start_response(s, 200, "OK", headers_names, headers_vxml, 3);
-            s->write(s, JK_STATUS_XMLH, sizeof(JK_STATUS_XMLH) - 1);
-            dump_config(s, p->s_worker, l);
-            s->write(s, JK_STATUS_XMLE, sizeof(JK_STATUS_XMLE) - 1);
-        }
-        else {
-            s->start_response(s, 200, "OK", headers_names, headers_vtxt, 3);
-            s->write(s,  JK_STATUS_TEXTUPDATE_RESPONCE,
-                     sizeof(JK_STATUS_TEXTUPDATE_RESPONCE) - 1);
-        }
-        if (worker)
-            free(worker);
-        JK_TRACE_EXIT(l);
-        return JK_TRUE;
+
+        s->write(s, JK_STATUS_BEND, sizeof(JK_STATUS_BEND) - 1);
+
     }
-
-    jk_log(l, JK_LOG_ERROR, "status: end of service with error");
+    else if (mime == 1) {
+        s->start_response(s, 200, "OK", headers_names, headers_vxml, 3);
+        s->write(s, JK_STATUS_XMLH, sizeof(JK_STATUS_XMLH) - 1);
+        dump_config(s, p->s_worker, l);
+        s->write(s, JK_STATUS_XMLE, sizeof(JK_STATUS_XMLE) - 1);
+    }
+    else {
+        s->start_response(s, 200, "OK", headers_names, headers_vtxt, 3);
+        s->write(s,  JK_STATUS_TEXTUPDATE_RESPONCE,
+                 sizeof(JK_STATUS_TEXTUPDATE_RESPONCE) - 1);
+    }
+    if (worker)
+        free(worker);
     JK_TRACE_EXIT(l);
-    return JK_FALSE;
+    return JK_TRUE;
 }
 
 static int JK_METHOD done(jk_endpoint_t **e, jk_logger_t *l)
