@@ -19,7 +19,7 @@
  *                                                                         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 386282 $                                          *
+ * Version:     $Revision: 437379 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -365,24 +365,45 @@ int uri_worker_map_open(jk_uri_worker_map_t *uw_map,
         if (sz > 0) {
             int i;
             for (i = 0; i < sz; i++) {
-                if (uri_worker_map_add
-                    (uw_map, jk_map_name_at(init_data, i),
-                     jk_map_value_at(init_data, i), l) == JK_FALSE) {
+                const char *u = jk_map_name_at(init_data, i);
+                const char *w = jk_map_value_at(init_data, i);
+                /* Multiple mappings like :
+                 * /servlets-examples|/ *
+                 * will create two mappings:
+                 * /servlets-examples
+                 * and:
+                 * /servlets-examples/ *
+                 */
+                if (strchr(u, '|')) {
+                    char *s, *r = strdup(u);
+                    s = strchr(r, '|');
+                    *(s++) = '\0';
+                    /* Add first mapping */
+                    if (!uri_worker_map_add(uw_map, r, w, l)) {
+                        jk_log(l, JK_LOG_ERROR,
+                        "invalid mapping rule %s->%s", r, w);
+                        rc = JK_FALSE;
+                    }
+                    for (; *s; s++)
+                        *(s - 1) = *s;
+                    *(s - 1) = '\0';
+                    /* add second mapping */
+                    if (!uri_worker_map_add(uw_map, r, w, l)) {
+                        jk_log(l, JK_LOG_ERROR,
+                               "invalid mapping rule %s->%s", r, w);
+                        rc = JK_FALSE;
+                    }
+                    free(r);
+                }
+                else if (!uri_worker_map_add(uw_map, u, w, l)) {
+                    jk_log(l, JK_LOG_ERROR,
+                           "invalid mapping rule %s->%s",
+                           u, w);
                     rc = JK_FALSE;
                     break;
                 }
-            }
-
-            if (i == sz) {
-                if (JK_IS_DEBUG_LEVEL(l))
-                    jk_log(l, JK_LOG_DEBUG,
-                           "there are %d rules",
-                           uw_map->size);
-            }
-            else {
-                jk_log(l, JK_LOG_ERROR,
-                       "Parsing error");
-                rc = JK_FALSE;
+                if (rc == JK_FALSE)
+                    break;
             }
         }
 
@@ -419,7 +440,7 @@ static int is_nomap_match(jk_uri_worker_map_t *uw_map,
             /* Map is already sorted by context_len */
             if (wildchar_match(uri, uwr->context,
 #ifdef WIN32
-                               1
+                               0
 #else
                                0
 #endif
@@ -454,10 +475,10 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
     unsigned int i;
     char *url_rewrite;
     const char *rv = NULL;
-    const char *url = uri;
-    char  buf[JK_MAX_URI_LEN+1];
+    char  url[JK_MAX_URI_LEN+1];
 
     JK_TRACE_ENTER(l);
+
     if (!uw_map || !uri) {
         JK_LOG_NULL_PARAMS(l);
         JK_TRACE_EXIT(l);
@@ -469,14 +490,19 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
         JK_TRACE_EXIT(l);
         return NULL;
     }
+    for (i = 0; i < strlen(uri); i++) 
+        if (uri[i] == ';')
+            break;
+        else
+            url[i] = uri[i];
+    url[i] = '\0';
+    
     url_rewrite = strstr(uri, JK_PATH_SESSION_IDENTIFIER);
     if (url_rewrite) {
-        size_t len = url_rewrite - uri;
+        size_t len = url_rewrite - url;
         if (len > JK_MAX_URI_LEN)
             len = JK_MAX_URI_LEN;
-        strncpy(buf, uri, len);
-        buf[len] = '\0';
-        url = &buf[0];
+        url[len] = '\0';
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG, "Removing Session path '%s' URI '%s'",
                    url_rewrite, url);
@@ -503,7 +529,7 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
             /* Map is already sorted by context_len */
             if (wildchar_match(url, uwr->context,
 #ifdef WIN32
-                               1
+                               0
 #else
                                0
 #endif
@@ -554,7 +580,7 @@ int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
 
     jk_map_alloc(&map);
     if (jk_map_read_properties(map, uw_map->fname,
-                               &uw_map->modified)) {
+                               &uw_map->modified, l)) {
         int i;
         for (i = 0; i < jk_map_size(map); i++) {
             const char *u = jk_map_name_at(map, i);
