@@ -18,7 +18,8 @@
  * Description: Utility functions (mainly configuration)                   *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 386592 $                                          *
+ * Author:      Rainer Jung <rjung@apache.org>                             *
+ * Version:     $Revision: 440635 $                                          *
  ***************************************************************************/
 
 
@@ -33,7 +34,6 @@
 #define STDERR_OF_WORKER            ("stderr")
 #define STDOUT_OF_WORKER            ("stdout")
 #define SECRET_OF_WORKER            ("secret")
-#define CONF_OF_WORKER              ("conf")
 #define MX_OF_WORKER                ("mx")
 #define MS_OF_WORKER                ("ms")
 #define CP_OF_WORKER                ("class_path")
@@ -47,7 +47,9 @@
 #define TYPE_OF_WORKER              ("type")
 #define CACHE_OF_WORKER_DEPRECATED  ("cachesize")
 #define CACHE_OF_WORKER             ("connection_pool_size")
-#define CACHE_TIMEOUT_OF_WORKER     ("cache_timeout")
+#define CACHE_OF_WORKER_MIN         ("connection_pool_minsize")
+#define CACHE_TIMEOUT_DEPRECATED    ("cache_timeout")
+#define CACHE_TIMEOUT_OF_WORKER     ("connection_pool_timeout")
 #define RECOVERY_OPTS_OF_WORKER     ("recovery_options")
 #define CONNECT_TIMEOUT_OF_WORKER   ("connect_timeout")
 #define PREPOST_TIMEOUT_OF_WORKER   ("prepost_timeout")
@@ -55,21 +57,24 @@
 #define SOCKET_TIMEOUT_OF_WORKER    ("socket_timeout")
 #define SOCKET_BUFFER_OF_WORKER     ("socket_buffer")
 #define SOCKET_KEEPALIVE_OF_WORKER  ("socket_keepalive")
-#define RECYCLE_TIMEOUT_OF_WORKER   ("recycle_timeout")
+#define RECYCLE_TIMEOUT_DEPRECATED  ("recycle_timeout")
 #define LOAD_FACTOR_OF_WORKER       ("lbfactor")
-/* deprecated directive. Use balance_workers instead */
-#define BALANCED_WORKERS            ("balanced_workers")
+#define DISTANCE_OF_WORKER          ("distance")
+#define BALANCED_WORKERS_DEPRECATED ("balanced_workers")
 #define BALANCE_WORKERS             ("balance_workers")
 #define STICKY_SESSION              ("sticky_session")
 #define STICKY_SESSION_FORCE        ("sticky_session_force")
+#define JVM_ROUTE_OF_WORKER         ("jvm_route")
 #define DOMAIN_OF_WORKER            ("domain")
 #define REDIRECT_OF_WORKER          ("redirect")
 #define MOUNT_OF_WORKER             ("mount")
 #define METHOD_OF_WORKER            ("method")
 #define LOCK_OF_WORKER              ("lock")
-#define IS_WORKER_DISABLED          ("disabled")
-#define IS_WORKER_STOPPED           ("stopped")
+#define IS_WORKER_DISABLED_DEPRECATED ("disabled")
+#define IS_WORKER_STOPPED_DEPRECATED  ("stopped")
+#define ACTIVATION_OF_WORKER        ("activation")
 #define WORKER_RECOVER_TIME         ("recover_time")
+#define WORKER_MAX_PACKET_SIZE      ("max_packet_size")
 
 
 #define DEFAULT_WORKER_TYPE         JK_AJP13_WORKER_NAME
@@ -81,7 +86,7 @@
 #define WORKER_MAINTAIN_PROPERTY_NAME ("worker.maintain")
 #define DEFAULT_MAINTAIN_TIME       (60)
 #define DEFAULT_LB_FACTOR           (1)
-#define LOG_FORMAT                  ("log_format")
+#define DEFAULT_DISTANCE            (0)
 
 #define TOMCAT32_BRIDGE_NAME        ("tomcat32")
 #define TOMCAT33_BRIDGE_NAME        ("tomcat33")
@@ -114,11 +119,76 @@
     extern long _ftol2(double dblSource) { return _ftol(dblSource); }
 #endif
 
-static const char *jk_level_werbs[] = {
+static const char *list_properties[] = {
+    BALANCE_WORKERS,
+    MOUNT_OF_WORKER,
+    "list",
+    NULL
+};
+
+static const char *unique_properties[] = {
+    SECRET_OF_WORKER,
+    HOST_OF_WORKER,
+    PORT_OF_WORKER,
+    TYPE_OF_WORKER,
+    CACHE_OF_WORKER_DEPRECATED,
+    CACHE_OF_WORKER,
+    CACHE_OF_WORKER_MIN,
+    CACHE_TIMEOUT_DEPRECATED,
+    CACHE_TIMEOUT_OF_WORKER,
+    RECOVERY_OPTS_OF_WORKER,
+    CONNECT_TIMEOUT_OF_WORKER,
+    PREPOST_TIMEOUT_OF_WORKER,
+    REPLY_TIMEOUT_OF_WORKER,
+    SOCKET_TIMEOUT_OF_WORKER,
+    SOCKET_BUFFER_OF_WORKER,
+    SOCKET_KEEPALIVE_OF_WORKER,
+    RECYCLE_TIMEOUT_DEPRECATED,
+    LOAD_FACTOR_OF_WORKER,
+    STICKY_SESSION,
+    STICKY_SESSION_FORCE,
+    JVM_ROUTE_OF_WORKER,
+    DOMAIN_OF_WORKER,
+    REDIRECT_OF_WORKER,
+    METHOD_OF_WORKER,
+    LOCK_OF_WORKER,
+    IS_WORKER_DISABLED_DEPRECATED,
+    IS_WORKER_STOPPED_DEPRECATED,
+    ACTIVATION_OF_WORKER,
+    WORKER_RECOVER_TIME,
+    WORKER_MAX_PACKET_SIZE,
+    RETRIES_OF_WORKER,
+    WORKER_MAINTAIN_PROPERTY_NAME,
+    NULL
+};
+
+static const char *deprecated_properties[] = {
+    SYSPROPS_OF_WORKER,
+    STDERR_OF_WORKER,
+    STDOUT_OF_WORKER,
+    MX_OF_WORKER,
+    MS_OF_WORKER,
+    CP_OF_WORKER,
+    BRIDGE_OF_WORKER,
+    JVM_OF_WORKER,
+    LIBPATH_OF_WORKER,
+    CMD_LINE_OF_WORKER,
+    NATIVE_LIB_OF_WORKER,
+    CACHE_OF_WORKER_DEPRECATED,
+    CACHE_TIMEOUT_DEPRECATED,
+    RECYCLE_TIMEOUT_DEPRECATED,
+    BALANCED_WORKERS_DEPRECATED,
+    IS_WORKER_DISABLED_DEPRECATED,
+    IS_WORKER_STOPPED_DEPRECATED,
+    NULL
+};
+
+/* All entries need to have fixed length 8 chars! */
+static const char *jk_level_verbs[] = {
     "[" JK_LOG_TRACE_VERB "] ",
     "[" JK_LOG_DEBUG_VERB "] ",
-    "[" JK_LOG_INFO_VERB  "]  ",
-    "[" JK_LOG_WARNING_VERB  "]  ",
+    "[" JK_LOG_INFO_VERB "]  ",
+    "[" JK_LOG_WARN_VERB "]  ",
     "[" JK_LOG_ERROR_VERB "] ",
     "[" JK_LOG_EMERG_VERB "] ",
     NULL
@@ -126,13 +196,32 @@ static const char *jk_level_werbs[] = {
 
 const char *jk_log_fmt = JK_TIME_FORMAT;
 
-static void set_time_str(char *str, int len)
+/* Sleep for 100ms */
+void jk_sleep(int ms)
+{
+#ifdef OS2
+    DosSleep(ms);
+#elif defined(BEOS)
+    snooze(ms * 1000);
+#elif defined(NETWARE)
+    delay(ms);
+#elif defined(WIN32)
+    Sleep(ms);
+#else
+    struct timeval tv;
+    tv.tv_usec = ms * 1000;
+    tv.tv_sec = 0;
+    select(0, NULL, NULL, NULL, &tv);
+#endif
+}
+
+static int set_time_str(char *str, int len)
 {
     time_t t = time(NULL);
     struct tm *tms;
 
     tms = localtime(&t);
-    strftime(str, len, jk_log_fmt, tms);
+    return (int)strftime(str, len, jk_log_fmt, tms);
 }
 
 /* Write at most n characters to the buffer in str, return the
@@ -183,11 +272,10 @@ static int JK_METHOD log_to_file(jk_logger_t *l, int level, const char *what)
         size_t sz = strlen(what);
         if (sz) {
             file_logger_t *p = l->logger_private;
-            fwrite(what, 1, sz, p->logfile);
-            fputc('\n', p->logfile);
-            /* [V] Flush the dam' thing! */
-            if (l->level < JK_LOG_INFO_LEVEL)
+            if (fwrite(what, 1, sz, p->logfile)) {
+                /* [V] Flush the dam' thing! */
                 fflush(p->logfile);
+            }
         }
 
         return JK_TRUE;
@@ -210,7 +298,7 @@ int jk_parse_log_level(const char *level)
         return JK_LOG_INFO_LEVEL;
     }
 
-    if (0 == strcasecmp(level, JK_LOG_WARNING_VERB)) {
+    if (0 == strcasecmp(level, JK_LOG_WARN_VERB)) {
         return JK_LOG_WARNING_LEVEL;
     }
 
@@ -222,7 +310,7 @@ int jk_parse_log_level(const char *level)
         return JK_LOG_EMERG_LEVEL;
     }
 
-    return JK_LOG_INFO_LEVEL;
+    return JK_LOG_DEF_LEVEL;
 }
 
 int jk_open_file_logger(jk_logger_t **l, const char *file, int level)
@@ -278,21 +366,23 @@ int jk_log(jk_logger_t *l,
            const char *fmt, ...)
 {
     int rc = 0;
+    /* Need to reserve space for newline and terminating zero byte. */
+    static int usable_size = HUGE_BUFFER_SIZE-2;
     if (!l || !file || !fmt) {
         return -1;
     }
 
     if ((l->level <= level) || (level == JK_LOG_REQUEST_LEVEL)) {
 #ifdef NETWARE
-/* On NetWare, this can get called on a thread that has a limited stack so */
-/* we will allocate and free the temporary buffer in this function         */
+        /* On NetWare, this can get called on a thread that has a limited stack so */
+        /* we will allocate and free the temporary buffer in this function         */
         char *buf;
 #else
         char buf[HUGE_BUFFER_SIZE];
 #endif
         char *f = (char *)(file + strlen(file) - 1);
         va_list args;
-        size_t used = 0;
+        int used = 0;
 
         while (f != file && '\\' != *f && '/' != *f) {
             f--;
@@ -306,51 +396,61 @@ int jk_log(jk_logger_t *l,
         if (NULL == buf)
             return -1;
 #endif
-        set_time_str(buf, HUGE_BUFFER_SIZE);
-        used = strlen(buf);
+        used = set_time_str(buf, usable_size);
 
-        /* Log [pid:threadid] for debug and trace levels */
-        if (l->level < JK_LOG_INFO_LEVEL) {
+        if (line) {
+            /* Log [pid:threadid] for all levels except REQUEST. */
+            /* This information helps to correlate lines from different logs. */
+            /* Performance is no issue, because with production log levels */
+            /* we only call it often, if we have a lot of errors */
 #ifdef USE_SPRINTF              /* until we get a snprintf function */
-            used += sprintf(&buf[used], "[%04d:%04d] ", getpid(),
+            rc = sprintf(&buf[used], "[%04d:%04d] ", getpid(),
                             jk_gettid());
 #else
-            used += snprintf(&buf[used], HUGE_BUFFER_SIZE - used,
+            rc = snprintf(&buf[used], usable_size - used,
                              "[%04d:%04d] ", getpid(), jk_gettid());
 #endif
-            if (used < 0) {
+            used += rc;
+            if (rc < 0 || usable_size - used < 8) {
                 return 0;
             }
-        }
-        if (line) {
-            strcat(buf, jk_level_werbs[level]);
+            strcat(buf, jk_level_verbs[level]);
             used += 8;
 
             if (funcname) {
-                strcat(buf, funcname);
-                strcat(buf, "::");
-                used += strlen(funcname) + 2;
+                rc = (int)strlen(funcname) + 2;
+                if (usable_size - used >= rc) {
+                    strcat(buf, funcname);
+                    strcat(buf, "::");
+                    used += rc;
+                }
             }
-        }
 
 #ifdef USE_SPRINTF              /* until we get a snprintf function */
-        if (line)
-            used += sprintf(&buf[used], "%s (%d): ", f, line);
+            rc = sprintf(&buf[used], "%s (%d): ", f, line);
 #else
-        if (line)
-            used += snprintf(&buf[used], HUGE_BUFFER_SIZE - used,
+            rc = snprintf(&buf[used], usable_size - used,
                              "%s (%d): ", f, line);
 #endif
-        if (used < 0) {
-            return 0;           /* [V] not sure what to return... */
+            used += rc;
+            if (rc < 0 || usable_size - used < 0) {
+                return 0;           /* [V] not sure what to return... */
+            }
         }
 
         va_start(args, fmt);
 #ifdef USE_VSPRINTF             /* until we get a vsnprintf function */
         rc = vsprintf(buf + used, fmt, args);
 #else
-        rc = vsnprintf(buf + used, HUGE_BUFFER_SIZE - used, fmt, args);
+        rc = vsnprintf(buf + used, usable_size - used, fmt, args);
 #endif
+        if ( rc <= usable_size - used ) {
+            used += rc;
+        } else {
+            used = usable_size;
+        }
+        buf[used] = '\n';
+        buf[used+1] = 0;
         va_end(args);
         l->log(l, level, buf);
 #ifdef NETWARE
@@ -372,6 +472,16 @@ const char *jk_get_worker_type(jk_map_t *m, const char *wname)
     return jk_map_get_string(m, buf, DEFAULT_WORKER_TYPE);
 }
 
+const char *jk_get_worker_jvm_route(jk_map_t *m, const char *wname, const char *def)
+{
+    char buf[1024];
+    if (!m || !wname) {
+        return NULL;
+    }
+    MAKE_WORKER_PARAM(JVM_ROUTE_OF_WORKER);
+    return jk_map_get_string(m, buf, def);
+}
+
 const char *jk_get_worker_domain(jk_map_t *m, const char *wname, const char *def)
 {
     char buf[1024];
@@ -500,6 +610,18 @@ int jk_get_worker_cache_size(jk_map_t *m, const char *wname, int def)
     return jk_map_get_int(m, buf, def);
 }
 
+int jk_get_worker_cache_size_min(jk_map_t *m, const char *wname, int def)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return -1;
+    }
+
+    MAKE_WORKER_PARAM(CACHE_OF_WORKER_MIN);
+    return jk_map_get_int(m, buf, def);
+}
+
 int jk_get_worker_socket_timeout(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
@@ -558,12 +680,16 @@ int jk_get_worker_socket_keepalive(jk_map_t *m, const char *wname, int def)
 int jk_get_worker_cache_timeout(jk_map_t *m, const char *wname, int def)
 {
     char buf[1024];
+    int rv;
 
     if (!m || !wname) {
         return -1;
     }
 
     MAKE_WORKER_PARAM(CACHE_TIMEOUT_OF_WORKER);
+    if ((rv = jk_map_get_int(m, buf, -1)) >= 0)
+        return rv;
+    MAKE_WORKER_PARAM(CACHE_TIMEOUT_DEPRECATED);
 
     return jk_map_get_int(m, buf, def);
 }
@@ -609,15 +735,7 @@ int jk_get_worker_reply_timeout(jk_map_t *m, const char *wname, int def)
 
 int jk_get_worker_recycle_timeout(jk_map_t *m, const char *wname, int def)
 {
-    char buf[1024];
-
-    if (!m || !wname) {
-        return -1;
-    }
-
-    MAKE_WORKER_PARAM(RECYCLE_TIMEOUT_OF_WORKER);
-
-    return jk_map_get_int(m, buf, def);
+    return def;
 }
 
 int jk_get_worker_retries(jk_map_t *m, const char *wname, int def)
@@ -686,7 +804,7 @@ int jk_get_is_worker_disabled(jk_map_t *m, const char *wname)
     char buf[1024];
     if (m && wname) {
         int value;
-        MAKE_WORKER_PARAM(IS_WORKER_DISABLED);
+        MAKE_WORKER_PARAM(IS_WORKER_DISABLED_DEPRECATED);
         value = jk_map_get_bool(m, buf, 0);
         if (!value)
             rc = JK_FALSE;
@@ -700,7 +818,7 @@ int jk_get_is_worker_stopped(jk_map_t *m, const char *wname)
     char buf[1024];
     if (m && wname) {
         int value;
-        MAKE_WORKER_PARAM(IS_WORKER_STOPPED);
+        MAKE_WORKER_PARAM(IS_WORKER_STOPPED_DEPRECATED);
         value = jk_map_get_bool(m, buf, 0);
         if (!value)
             rc = JK_FALSE;
@@ -708,6 +826,34 @@ int jk_get_is_worker_stopped(jk_map_t *m, const char *wname)
     return rc;
 }
 
+int jk_get_worker_activation(jk_map_t *m, const char *wname)
+{
+    char buf[1024];
+    const char *v;
+    if (!m || !wname) {
+        return JK_LB_ACTIVATION_ACTIVE;
+    }
+
+    MAKE_WORKER_PARAM(ACTIVATION_OF_WORKER);
+    v = jk_map_get_string(m, buf, NULL);
+    if (v) {
+        if (*v == 'a' || *v == 'A')
+            return JK_LB_ACTIVATION_ACTIVE;
+        else if (*v == 's' || *v == 'S')
+            return JK_LB_ACTIVATION_STOPPED;
+        else if (*v == 'd' || *v == 'D')
+            return JK_LB_ACTIVATION_DISABLED;
+        else
+            return JK_LB_ACTIVATION_ACTIVE;
+    }
+    else if (jk_get_is_worker_stopped(m, wname))
+        return JK_LB_ACTIVATION_STOPPED;
+    else if (jk_get_is_worker_disabled(m, wname))
+        return JK_LB_ACTIVATION_DISABLED;
+    else
+        return JK_LB_ACTIVATION_ACTIVE;
+}
+
 void jk_set_log_format(const char *logformat)
 {
     jk_log_fmt = (logformat) ? logformat : JK_TIME_FORMAT;
@@ -726,6 +872,19 @@ int jk_get_lb_factor(jk_map_t *m, const char *wname)
     return jk_map_get_int(m, buf, DEFAULT_LB_FACTOR);
 }
 
+int jk_get_distance(jk_map_t *m, const char *wname)
+{
+    char buf[1024];
+
+    if (!m || !wname) {
+        return DEFAULT_DISTANCE;
+    }
+
+    MAKE_WORKER_PARAM(DISTANCE_OF_WORKER);
+
+    return jk_map_get_int(m, buf, DEFAULT_DISTANCE);
+}
+
 int jk_get_is_sticky_session(jk_map_t *m, const char *wname)
 {
     int rc = JK_TRUE;
@@ -759,21 +918,21 @@ int jk_get_lb_method(jk_map_t *m, const char *wname)
     char buf[1024];
     const char *v;
     if (!m || !wname) {
-        return DEFAULT_LB_FACTOR;
+        return JK_LB_METHOD_DEF;
     }
 
     MAKE_WORKER_PARAM(METHOD_OF_WORKER);
     v = jk_map_get_string(m, buf, NULL);
     if (!v)
-        return JK_LB_BYREQUESTS;
+        return JK_LB_METHOD_DEF;
     else if  (*v == 't' || *v == 'T' || *v == '1')
-        return JK_LB_BYTRAFFIC;
+        return JK_LB_METHOD_TRAFFIC;
     else if  (*v == 'r' || *v == 'R' || *v == '0')
-        return JK_LB_BYREQUESTS;
+        return JK_LB_METHOD_REQUESTS;
     else if  (*v == 'b' || *v == 'B' || *v == '2')
-        return JK_LB_BYBUSYNESS;
+        return JK_LB_METHOD_BUSYNESS;
     else
-        return JK_LB_BYREQUESTS;
+        return JK_LB_METHOD_DEF;
 }
 
 int jk_get_lb_lock(jk_map_t *m, const char *wname)
@@ -781,19 +940,39 @@ int jk_get_lb_lock(jk_map_t *m, const char *wname)
     char buf[1024];
     const char *v;
     if (!m || !wname) {
-        return JK_LB_LOCK_DEFAULT;
+        return JK_LB_LOCK_DEF;
     }
 
     MAKE_WORKER_PARAM(LOCK_OF_WORKER);
     v = jk_map_get_string(m, buf, NULL);
     if (!v)
-        return JK_LB_LOCK_DEFAULT;
+        return JK_LB_LOCK_DEF;
     else if  (*v == 'o' || *v == 'O' || *v == '0')
-        return JK_LB_LOCK_DEFAULT;
+        return JK_LB_LOCK_OPTIMISTIC;
     else if  (*v == 'p' || *v == 'P' || *v == '1')
         return JK_LB_LOCK_PESSIMISTIC;
     else
-        return JK_LB_LOCK_DEFAULT;
+        return JK_LB_LOCK_DEF;
+}
+
+int jk_get_max_packet_size(jk_map_t *m, const char *wname)
+{
+    char buf[1024];
+    int sz;
+
+    if (!m || !wname) {
+        return DEF_BUFFER_SZ;
+    }
+
+    MAKE_WORKER_PARAM(DISTANCE_OF_WORKER);
+    sz = jk_map_get_int(m, buf, DEF_BUFFER_SZ);
+    sz = JK_ALIGN(sz, 1024);
+    if (sz < DEF_BUFFER_SZ)
+        sz = DEF_BUFFER_SZ;
+    else if (sz > 64*1024)
+        sz = 64*1024;
+    
+    return sz;
 }
 
 int jk_get_lb_worker_list(jk_map_t *m,
@@ -812,7 +991,7 @@ int jk_get_lb_worker_list(jk_map_t *m,
             return JK_TRUE;
         }
         /* Try old balanced_workers directive */
-        MAKE_WORKER_PARAM(BALANCED_WORKERS);
+        MAKE_WORKER_PARAM(BALANCED_WORKERS_DEPRECATED);
         ar = jk_map_get_string_list(m, buf, num_of_wokers, NULL);
         if (ar) {
             *list = ar;
@@ -999,14 +1178,20 @@ int jk_file_exists(const char *f)
     return JK_FALSE;
 }
 
-static int jk_is_some_property(const char *prp_name, const char *suffix)
+static int jk_is_some_property(const char *prp_name, const char *suffix, const char *sep)
 {
+    char buf[1024];
+
     if (prp_name && suffix) {
-        size_t prp_name_len = strlen(prp_name);
-        size_t suffix_len = strlen(suffix);
+        size_t prp_name_len;
+        size_t suffix_len;
+        strcpy(buf, sep);
+        strcat(buf, suffix);
+        prp_name_len = strlen(prp_name);
+        suffix_len = strlen(buf);
         if (prp_name_len >= suffix_len) {
             const char *prp_suffix = prp_name + prp_name_len - suffix_len;
-            if (0 == strcmp(suffix, prp_suffix)) {
+            if (0 == strcmp(buf, prp_suffix)) {
                 return JK_TRUE;
             }
         }
@@ -1015,51 +1200,43 @@ static int jk_is_some_property(const char *prp_name, const char *suffix)
     return JK_FALSE;
 }
 
-int jk_is_path_poperty(const char *prp_name)
+int jk_is_path_property(const char *prp_name)
 {
-    return jk_is_some_property(prp_name, "path");
+    return jk_is_some_property(prp_name, "path", "_");
 }
 
-int jk_is_cmd_line_poperty(const char *prp_name)
+int jk_is_cmd_line_property(const char *prp_name)
 {
-    return jk_is_some_property(prp_name, CMD_LINE_OF_WORKER);
+    return jk_is_some_property(prp_name, CMD_LINE_OF_WORKER, ".");
 }
 
-static const char *unique_properties[] = {
-    HOST_OF_WORKER,
-    PORT_OF_WORKER,
-    TYPE_OF_WORKER,
-    CACHE_OF_WORKER,
-    CACHE_OF_WORKER_DEPRECATED,
-    CACHE_TIMEOUT_OF_WORKER,
-    RECOVERY_OPTS_OF_WORKER,
-    CONNECT_TIMEOUT_OF_WORKER,
-    PREPOST_TIMEOUT_OF_WORKER,
-    REPLY_TIMEOUT_OF_WORKER,
-    SOCKET_TIMEOUT_OF_WORKER,
-    SOCKET_BUFFER_OF_WORKER,
-    SOCKET_KEEPALIVE_OF_WORKER,
-    RECYCLE_TIMEOUT_OF_WORKER,
-    LOAD_FACTOR_OF_WORKER,
-    STICKY_SESSION,
-    STICKY_SESSION_FORCE,
-    DOMAIN_OF_WORKER,
-    REDIRECT_OF_WORKER,
-    MOUNT_OF_WORKER,
-    METHOD_OF_WORKER,
-    IS_WORKER_DISABLED,
-    IS_WORKER_STOPPED,
-    WORKER_RECOVER_TIME,
-    SECRET_OF_WORKER,
-    RETRIES_OF_WORKER,
-    NULL
-};
+int jk_is_list_property(const char *prp_name)
+{
+    const char **props = &list_properties[0];
+    while (*props) {
+        if (jk_is_some_property(prp_name, *props, "."))
+            return JK_TRUE;
+        props++;
+    }
+    return JK_FALSE;
+}
 
 int jk_is_unique_property(const char *prp_name)
 {
     const char **props = &unique_properties[0];
     while (*props) {
-        if (jk_is_some_property(prp_name, *props))
+        if (jk_is_some_property(prp_name, *props, "."))
+            return JK_TRUE;
+        props++;
+    }
+    return JK_FALSE;
+}
+
+int jk_is_deprecated_property(const char *prp_name)
+{
+    const char **props = &deprecated_properties[0];
+    while (*props) {
+        if (jk_is_some_property(prp_name, *props, "."))
             return JK_TRUE;
         props++;
     }
@@ -1232,6 +1409,7 @@ void jk_init_ws_service(jk_ws_service_t *s)
     s->num_attributes = 0;
     s->jvm_route = NULL;
     s->retries = JK_RETRIES;
+    s->add_log_items = NULL;
 }
 
 #ifdef _MT_CODE_PTHREAD
