@@ -20,7 +20,7 @@
  * Author:      Larry Isaacs <larryi@apache.org>                           *
  * Author:      Ignacio J. Ortega <nacho@apache.org>                       *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 375077 $                                          *
+ * Version:     $Revision: 438903 $                                          *
  ***************************************************************************/
 
 // This define is needed to include wincrypt,h, needed to get client certificates
@@ -78,6 +78,7 @@ static char HTTP_WORKER_HEADER_NAME[_MAX_FNAME];
 #define URI_SELECT_PARSED_VERB      ("parsed")
 #define URI_SELECT_UNPARSED_VERB    ("unparsed")
 #define URI_SELECT_ESCAPED_VERB     ("escaped")
+#define URI_REWRITE_VERB            ("rewrite_rule_file")
 
 #define TRANSLATE_HEADER                              ("Translate:")
 #define TRANSLATE_HEADER_NAME                         ("Translate")
@@ -137,6 +138,8 @@ static int iis5 = -1;
 
 static jk_uri_worker_map_t *uw_map = NULL;
 static jk_map_t *workers_map = NULL;
+static jk_map_t *rewrite_map = NULL;
+
 static jk_logger_t *logger = NULL;
 static char *SERVER_NAME = "SERVER_NAME";
 static char *SERVER_SOFTWARE = "SERVER_SOFTWARE";
@@ -145,9 +148,10 @@ static char *CONTENT_TYPE = "Content-Type:text/html\r\n\r\n";
 static char extension_uri[INTERNET_MAX_URL_LENGTH] =
     "/jakarta/isapi_redirect.dll";
 static char log_file[MAX_PATH * 2];
-static int log_level = JK_LOG_EMERG_LEVEL;
+static int log_level = JK_LOG_DEF_LEVEL;
 static char worker_file[MAX_PATH * 2];
 static char worker_mount_file[MAX_PATH * 2] = {0};
+static char rewrite_rule_file[MAX_PATH * 2] = {0};
 
 #define URI_SELECT_OPT_PARSED       0
 #define URI_SELECT_OPT_UNPARSED     1
@@ -681,6 +685,24 @@ BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
     return TRUE;
 }
 
+static int simple_rewrite(char *uri)
+{
+    if (rewrite_map) {
+        int i;
+        char buf[INTERNET_MAX_URL_LENGTH];
+        for (i = 0; i < jk_map_size(rewrite_map); i++) {
+            const char *src = jk_map_name_at(rewrite_map, i);
+            if (strncmp(uri, src, strlen(src)) == 0) {
+                strcpy(buf, jk_map_value_at(rewrite_map, i));
+                strcat(buf, uri + strlen(src));
+                strcpy(uri, buf);
+                return 1;
+            }
+        }
+    }
+    return 0;
+}
+
 DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                             DWORD dwNotificationType, LPVOID pvNotification)
 {
@@ -726,6 +748,7 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
         char Host[INTERNET_MAX_URL_LENGTH] = "";
         char Port[INTERNET_MAX_URL_LENGTH] = "";
         char Translate[INTERNET_MAX_URL_LENGTH];
+        char squery[INTERNET_MAX_URL_LENGTH] = "";
         BOOL(WINAPI * GetHeader)
             (struct _HTTP_FILTER_CONTEXT * pfc, LPSTR lpszName,
              LPVOID lpvBuffer, LPDWORD lpdwSize);
@@ -781,6 +804,7 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
             query = strchr(uri, '?');
             if (query) {
                 *query++ = '\0';
+                strcpy(squery, query);
             }
 
             rc = unescape_url(uri);
@@ -895,10 +919,28 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                 else {
                     forwardURI = uri;
                 }
+                /* Do a simple rewrite .
+                 * Note that URI can be escaped, so thus the rule has
+                 * to be in that case.
+                 *
+                 * TODO: Add more advanced regexp rewrite.
+                 */
+                if (JK_IS_DEBUG_LEVEL(logger)) {
+                    char duri[INTERNET_MAX_URL_LENGTH];
+                    strcpy(duri, forwardURI);
+                    if (simple_rewrite(forwardURI)) {
+                        jk_log(logger, JK_LOG_DEBUG,
+                               "rewriten URI [%s]->[%s]",
+                               duri, forwardURI);
+                    }
+                }
+                else {
+                    simple_rewrite(forwardURI);
+                }
 
                 if (!AddHeader(pfc, URI_HEADER_NAME, forwardURI) ||
-                    ((query != NULL && strlen(query) > 0)
-                     ? !AddHeader(pfc, QUERY_HEADER_NAME, query) : FALSE) ||
+                    ((strlen(squery) > 0)
+                     ? !AddHeader(pfc, QUERY_HEADER_NAME, squery) : FALSE) ||
                     !AddHeader(pfc, WORKER_HEADER_NAME, (LPSTR)worker) ||
                     !SetHeader(pfc, "url", extension_uri)) {
                     jk_log(logger, JK_LOG_ERROR,
@@ -932,15 +974,13 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                     }
                     memset(ld, 0, sizeof(isapi_log_data_t));
                     strcpy(ld->uri, forwardURI);
-                    if (query && strlen(query) > 0)
-                        strcpy(ld->query, query);
+                    strcpy(ld->query, squery);
                     pfc->pFilterContext = ld;
                 } else {
                     isapi_log_data_t *ld = (isapi_log_data_t *)pfc->pFilterContext;
                     memset(ld, 0, sizeof(isapi_log_data_t));
                     strcpy(ld->uri, forwardURI);
-                    if (query && strlen(query) > 0)
-                        strcpy(ld->query, query);
+                    strcpy(ld->query, squery);
                 }
             }
             else {
@@ -1172,8 +1212,24 @@ static int init_jk(char *serverName)
         jk_log(logger, JK_LOG_DEBUG, "Using worker file %s.", worker_file);
         jk_log(logger, JK_LOG_DEBUG, "Using worker mount file %s.",
                worker_mount_file);
+        jk_log(logger, JK_LOG_DEBUG, "Using rewrite rule file %s.",
+               rewrite_rule_file);
         jk_log(logger, JK_LOG_DEBUG, "Using uri select %d.", uri_select_option);
     }
+
+    if (rewrite_rule_file[0] && jk_map_alloc(&rewrite_map)) {
+        if (jk_map_read_properties(rewrite_map, rewrite_rule_file, NULL, logger)) {
+            if (JK_IS_DEBUG_LEVEL(logger)) {
+                jk_log(logger, JK_LOG_DEBUG, "Loaded rewrite rule file %s.",
+                       rewrite_rule_file);
+
+            }
+            else {
+                jk_map_free(&rewrite_map);
+            }
+        }
+    }
+
     if (uri_worker_map_alloc(&uw_map, NULL, logger)) {
         rc = JK_FALSE;
         uw_map->fname = worker_mount_file;
@@ -1183,7 +1239,7 @@ static int init_jk(char *serverName)
     if (rc) {
         rc = JK_FALSE;
         if (jk_map_alloc(&workers_map)) {
-            if (jk_map_read_properties(workers_map, worker_file, NULL)) {
+            if (jk_map_read_properties(workers_map, worker_file, NULL, logger)) {
                 /* we add the URI->WORKER MAP since workers using AJP14 will feed it */
 
                 worker_env.uri_to_worker = uw_map;
@@ -1243,7 +1299,7 @@ static int read_registry_init_data(void)
     jk_map_t *map;
 
     if (jk_map_alloc(&map)) {
-        if (jk_map_read_properties(map, ini_file_name, NULL)) {
+        if (jk_map_read_properties(map, ini_file_name, NULL, logger)) {
             using_ini_file = JK_TRUE;
         }
     }
@@ -1259,9 +1315,6 @@ static int read_registry_init_data(void)
         if (tmp) {
             log_level = jk_parse_log_level(tmp);
         }
-        else {
-            ok = JK_FALSE;
-        }
         tmp = jk_map_get_string(map, EXTENSION_URI_TAG, NULL);
         if (tmp) {
             strcpy(extension_uri, tmp);
@@ -1283,6 +1336,10 @@ static int read_registry_init_data(void)
         else {
             ok = JK_FALSE;
         }
+        tmp = jk_map_get_string(map, URI_REWRITE_VERB, NULL);
+        if (tmp) {
+            strcpy(rewrite_rule_file, tmp);
+        }
         tmp = jk_map_get_string(map, URI_SELECT_TAG, NULL);
         if (tmp) {
             int opt = parse_uri_select(tmp);
@@ -1316,9 +1373,6 @@ static int read_registry_init_data(void)
                                           tmpbuf, sizeof(tmpbuf))) {
             log_level = jk_parse_log_level(tmpbuf);
         }
-        else {
-            ok = JK_FALSE;
-        }
 
         if (get_registry_config_parameter(hkey,
                                           EXTENSION_URI_TAG,
@@ -1348,6 +1402,13 @@ static int read_registry_init_data(void)
             ok = JK_FALSE;
         }
 
+        if (get_registry_config_parameter(hkey,
+                                          URI_REWRITE_VERB,
+                                          tmpbuf,
+                                          sizeof(rewrite_rule_file))) {
+            strcpy(rewrite_rule_file, tmpbuf);
+        }
+
         if (get_registry_config_parameter(hkey,
                                           URI_SELECT_TAG,
                                           tmpbuf, sizeof(tmpbuf))) {
