@@ -22,7 +22,7 @@ import java.nio.ByteBuffer;
 /** Socket
  *
  * @author Mladen Turk
- * @version $Revision: 327657 $, $Date: 2005-10-22 08:32:32 -0400 (Sat, 22 Oct 2005) $
+ * @version $Revision: 416780 $, $Date: 2006-06-23 12:58:41 -0500 (Fri, 23 Jun 2006) $
  */
 
 public class Socket {
@@ -525,4 +525,22 @@ public class Socket {
      * @param buf The ByteBuffer
      */
     public static native void setrbb(long sock, ByteBuffer buf);
+
+    /**
+     * Set the data associated with the current socket.
+     * @param sock The currently open socket.
+     * @param data The user data to associate with the socket.
+     * @param key The key to associate with the data.
+     * @param cleanup The cleanup to call when the socket is destroyed.
+     */
+      public static native int dataSet(long sock, String key, Object data);
+
+    /**
+     * Return the data associated with the current socket
+     * @param data The user data associated with the socket.
+     * @param key The key to associate with the user data.
+     * @param sock The currently open socket.
+     * @return Data or null in case of error.
+     */
+     public static native Object dataGet(long sock, String key);
 }
