@@ -16,13 +16,15 @@
 /*
  *
  * @author Mladen Turk
- * @version $Revision: 388090 $, $Date: 2006-03-23 02:53:24 -0500 (Thu, 23 Mar 2006) $
+ * @version $Revision: 411550 $, $Date: 2006-06-04 07:06:09 -0500 (Sun, 04 Jun 2006) $
  */
 
 #ifndef _WIN32_WINNT
 #define _WIN32_WINNT 0x0500
 #endif
-#include <windows.h>
+#include <winsock2.h>
+#include <mswsock.h>
+#include <ws2tcpip.h>
 #include <shlwapi.h>
 
 #include "apr.h"
