@@ -1,4 +1,4 @@
-/* Copyright 2000-2005 The Apache Software Foundation
+/* Copyright 2000-2006 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,13 +16,15 @@
 /*
  *
  * @author Mladen Turk
- * @version $Revision: 390147 $, $Date: 2006-03-30 09:32:04 -0500 (Thu, 30 Mar 2006) $
+ * @version $Revision: 442529 $, $Date: 2006-09-12 03:54:10 -0500 (Tue, 12 Sep 2006) $
  */
 
 #include "tcn.h"
 
 #ifdef TCN_DO_STATISTICS
+
 #include "apr_atomic.h"
+
 static volatile apr_uint32_t sp_created  = 0;
 static volatile apr_uint32_t sp_closed   = 0;
 static volatile apr_uint32_t sp_cleared  = 0;
@@ -45,25 +47,6 @@ static volatile apr_size_t   sf_min_send = 10000000;
 static volatile apr_uint32_t sf_num_send = 0;
 static volatile apr_off_t    sf_tot_send = 0;
 
-#endif
-
-#if  !APR_HAVE_IPV6
-#define APR_INET6 APR_INET
-#endif
-
-#define GET_S_FAMILY(T, F)           \
-    if (F == 0) T = APR_UNSPEC;      \
-    else if (F == 1) T = APR_INET;   \
-    else if (F == 2) T = APR_INET6;  \
-    else T = F
-
-#define GET_S_TYPE(T, F)             \
-    if (F == 0) T = SOCK_STREAM;     \
-    else if (F == 1) T = SOCK_DGRAM; \
-    else T = F
-
-#ifdef TCN_DO_STATISTICS
-
 void sp_network_dump_statistics()
 {
     fprintf(stderr, "Network Statistics ......\n");
@@ -72,13 +55,13 @@ void sp_network_dump_statistics()
     fprintf(stderr, "Sockets closed          : %d\n", sp_closed);
     fprintf(stderr, "Sockets cleared         : %d\n", sp_cleared);
     fprintf(stderr, "Total send calls        : %d\n", sp_num_send);
-    fprintf(stderr, "Minimum send lenght     : %d\n", sp_min_send);
-    fprintf(stderr, "Maximum send lenght     : %d\n", sp_max_send);
-    fprintf(stderr, "Average send lenght     : %.2f\n", (double)sp_tot_send/(double)sp_num_send);
+    fprintf(stderr, "Minimum send length     : %d\n", sp_min_send);
+    fprintf(stderr, "Maximum send length     : %d\n", sp_max_send);
+    fprintf(stderr, "Average send length     : %.2f\n", (double)sp_tot_send/(double)sp_num_send);
     fprintf(stderr, "Total recv calls        : %d\n", sp_num_recv);
-    fprintf(stderr, "Minimum recv lenght     : %d\n", sp_min_recv);
-    fprintf(stderr, "Maximum recv lenght     : %d\n", sp_max_recv);
-    fprintf(stderr, "Average recv lenght     : %.2f\n", (double)sp_tot_recv/(double)sp_num_recv);
+    fprintf(stderr, "Minimum recv length     : %d\n", sp_min_recv);
+    fprintf(stderr, "Maximum recv length     : %d\n", sp_max_recv);
+    fprintf(stderr, "Average recv length     : %.2f\n", (double)sp_tot_recv/(double)sp_num_recv);
     fprintf(stderr, "Receive timeouts        : %d\n", sp_tmo_recv);
     fprintf(stderr, "Receive errors          : %d\n", sp_err_recv);
     fprintf(stderr, "Receive resets          : %d\n", sp_rst_recv);
@@ -90,90 +73,7 @@ void sp_network_dump_statistics()
 
 }
 
-#endif
-
-TCN_IMPLEMENT_CALL(jlong, Address, info)(TCN_STDARGS,
-                                         jstring hostname,
-                                         jint family, jint port,
-                                         jint flags, jlong pool)
-{
-    apr_pool_t *p = J2P(pool, apr_pool_t *);
-    TCN_ALLOC_CSTRING(hostname);
-    apr_sockaddr_t *sa = NULL;
-    apr_int32_t f;
-
-
-    UNREFERENCED(o);
-    GET_S_FAMILY(f, family);
-    TCN_THROW_IF_ERR(apr_sockaddr_info_get(&sa,
-            J2S(hostname), f, (apr_port_t)port,
-            (apr_int32_t)flags, p), sa);
-
-cleanup:
-    TCN_FREE_CSTRING(hostname);
-    return P2J(sa);
-}
-
-TCN_IMPLEMENT_CALL(jstring, Address, getnameinfo)(TCN_STDARGS,
-                                                  jlong sa, jint flags)
-{
-    apr_sockaddr_t *s = J2P(sa, apr_sockaddr_t *);
-    char *hostname;
-
-    UNREFERENCED(o);
-    if (apr_getnameinfo(&hostname, s, (apr_int32_t)flags) == APR_SUCCESS)
-        return AJP_TO_JSTRING(hostname);
-    else
-        return NULL;
-}
-
-TCN_IMPLEMENT_CALL(jstring, Address, getip)(TCN_STDARGS, jlong sa)
-{
-    apr_sockaddr_t *s = J2P(sa, apr_sockaddr_t *);
-    char *ipaddr;
-
-    UNREFERENCED(o);
-    if (apr_sockaddr_ip_get(&ipaddr, s) == APR_SUCCESS)
-        return AJP_TO_JSTRING(ipaddr);
-    else
-        return NULL;
-}
-
-TCN_IMPLEMENT_CALL(jlong, Address, get)(TCN_STDARGS, jint which,
-                                        jlong sock)
-{
-    tcn_socket_t *s = J2P(sock, tcn_socket_t *);
-    apr_sockaddr_t *sa = NULL;
-
-    UNREFERENCED(o);
-    TCN_THROW_IF_ERR(apr_socket_addr_get(&sa,
-                        (apr_interface_e)which, s->sock), sa);
-cleanup:
-    return P2J(sa);
-}
-
-TCN_IMPLEMENT_CALL(jint, Address, equal)(TCN_STDARGS,
-                                         jlong a, jlong b)
-{
-    apr_sockaddr_t *sa = J2P(a, apr_sockaddr_t *);
-    apr_sockaddr_t *sb = J2P(b, apr_sockaddr_t *);
-
-    UNREFERENCED_STDARGS;
-    return apr_sockaddr_equal(sa, sb) ? JNI_TRUE : JNI_FALSE;
-}
-
-TCN_IMPLEMENT_CALL(jint, Address, getservbyname)(TCN_STDARGS,
-                                                 jlong sa, jstring servname)
-{
-    apr_sockaddr_t *s = J2P(sa, apr_sockaddr_t *);
-    TCN_ALLOC_CSTRING(servname);
-    apr_status_t rv;
-
-    UNREFERENCED(o);
-    rv = apr_getservbyname(s, J2S(servname));
-    TCN_FREE_CSTRING(servname);
-    return (jint)rv;
-}
+#endif /* TCN_DO_STATISTICS */
 
 static apr_status_t sp_socket_cleanup(void *data)
 {
@@ -288,6 +188,7 @@ TCN_IMPLEMENT_CALL(jlong, Socket, create)(TCN_STDARGS, jint family,
     sp_created++;
 #endif
     a = (tcn_socket_t *)apr_pcalloc(p, sizeof(tcn_socket_t));
+    TCN_CHECK_ALLOCATED(a);
     a->sock = s;
     a->pool = p;
     if (family >= 0)
@@ -425,6 +326,7 @@ TCN_IMPLEMENT_CALL(jlong, Socket, acceptx)(TCN_STDARGS, jlong sock,
         apr_atomic_inc32(&sp_accepted);
 #endif
         a = (tcn_socket_t *)apr_pcalloc(p, sizeof(tcn_socket_t));
+        TCN_CHECK_ALLOCATED(a);
         a->sock   = n;
         a->pool   = p;
         a->net    = &apr_socket_layer;
@@ -462,6 +364,7 @@ TCN_IMPLEMENT_CALL(jlong, Socket, accept)(TCN_STDARGS, jlong sock)
         apr_atomic_inc32(&sp_accepted);
 #endif
         a = (tcn_socket_t *)apr_pcalloc(p, sizeof(tcn_socket_t));
+        TCN_CHECK_ALLOCATED(a);
         a->sock   = n;
         a->pool   = p;
         a->net    = &apr_socket_layer;
@@ -472,8 +375,6 @@ TCN_IMPLEMENT_CALL(jlong, Socket, accept)(TCN_STDARGS, jlong sock)
     }
     return P2J(a);
 cleanup:
-    if (p)
-        apr_pool_destroy(p);
     return 0;
 }
 
@@ -507,23 +408,17 @@ TCN_IMPLEMENT_CALL(jint, Socket, send)(TCN_STDARGS, jlong sock,
 #endif
 
     if (tosend <= TCN_BUFFER_SZ) {
-        char sb[TCN_BUFFER_SZ];
-        (*e)->GetByteArrayRegion(e, buf, offset, tosend, (jbyte *)sb);
+        jbyte sb[TCN_BUFFER_SZ];
+        (*e)->GetByteArrayRegion(e, buf, offset, tosend, &sb[0]);
         ss = (*s->net->send)(s->opaque, sb, &nbytes);
     }
     else {
-        jbyte *bytes;
-        apr_int32_t nb;
-        apr_socket_opt_get(s->sock, APR_SO_NONBLOCK, &nb);
-        if (nb)
-            bytes = (*e)->GetPrimitiveArrayCritical(e, buf, NULL);
-        else
-            bytes = (*e)->GetByteArrayElements(e, buf, NULL);
-        ss = (*s->net->send)(s->opaque, (char *)(bytes + offset), &nbytes);
-        if (nb)
-            (*e)->ReleasePrimitiveArrayCritical(e, buf, bytes, JNI_ABORT);
-        else
-            (*e)->ReleaseByteArrayElements(e, buf, bytes, JNI_ABORT);
+        jbyte *sb = (jbyte *)malloc(nbytes);
+        if (sb == NULL)
+            return -APR_ENOMEM;
+        (*e)->GetByteArrayRegion(e, buf, offset, tosend, sb);
+        ss = (*s->net->send)(s->opaque, sb, &nbytes);
+        free(sb);
     }
     if (ss == APR_SUCCESS)
         return (jint)nbytes;
@@ -724,9 +619,10 @@ TCN_IMPLEMENT_CALL(jint, Socket, recv)(TCN_STDARGS, jlong sock,
     }
     else {
         jbyte *bytes = (*e)->GetByteArrayElements(e, buf, NULL);
-        ss = (*s->net->recv)(s->opaque, (char*)(bytes + offset), &nbytes);
-        (*e)->ReleaseByteArrayElements(e, buf, bytes,
-                                       nbytes ? 0 : JNI_ABORT);
+        if ((ss = (*s->net->recv)(s->opaque, (char*)(bytes + offset),
+                                  &nbytes)) == APR_SUCCESS)
+            (*e)->ReleaseByteArrayElements(e, buf, bytes,
+                                           nbytes ? 0 : JNI_ABORT);
     }
 #ifdef TCN_DO_STATISTICS
     if (ss == APR_SUCCESS) {
@@ -764,30 +660,27 @@ TCN_IMPLEMENT_CALL(jint, Socket, recvt)(TCN_STDARGS, jlong sock,
     tcn_socket_t *s = J2P(sock, tcn_socket_t *);
     apr_size_t nbytes = (apr_size_t)toread;
     apr_status_t ss;
-    apr_interval_time_t t;
 
     UNREFERENCED(o);
     TCN_ASSERT(sock != 0);
     TCN_ASSERT(s->opaque != NULL);
     TCN_ASSERT(buf != NULL);
 
-    if ((ss = (*s->net->timeout_get)(s->opaque, &t)) != APR_SUCCESS)
-        goto cleanup;
     if ((ss = (*s->net->timeout_set)(s->opaque, J2T(timeout))) != APR_SUCCESS)
         goto cleanup;
     if (toread <= TCN_BUFFER_SZ) {
-        char sb[TCN_BUFFER_SZ];
-        ss = (*s->net->recv)(s->opaque, sb, &nbytes);
-        (*e)->SetByteArrayRegion(e, buf, offset, (jsize)nbytes, (jbyte*)&sb[0]);
+        jbyte sb[TCN_BUFFER_SZ];
+        if ((ss = (*s->net->recv)(s->opaque, sb, &nbytes)) == APR_SUCCESS)
+            (*e)->SetByteArrayRegion(e, buf, offset, (jsize)nbytes, &sb[0]);
     }
     else {
-        jbyte *bytes = (*e)->GetByteArrayElements(e, buf, NULL);
-        ss = (*s->net->recv)(s->opaque, (char*)(bytes + offset), &nbytes);
-        (*e)->ReleaseByteArrayElements(e, buf, bytes,
-                                       nbytes ? 0 : JNI_ABORT);
+        jbyte *sb = (jbyte *)malloc(nbytes);
+        if (sb == NULL)
+            return -APR_ENOMEM;
+        if ((ss = (*s->net->recv)(s->opaque, sb, &nbytes)) == APR_SUCCESS)
+            (*e)->SetByteArrayRegion(e, buf, offset, (jsize)nbytes, &sb[0]);
+        free(sb);
     }
-    /* Resore the original timeout */
-    (*s->net->timeout_set)(s->opaque, t);
 #ifdef TCN_DO_STATISTICS
     if (ss == APR_SUCCESS) {
         sp_max_recv = TCN_MAX(sp_max_recv, nbytes);
@@ -913,7 +806,6 @@ TCN_IMPLEMENT_CALL(jint, Socket, recvbt)(TCN_STDARGS, jlong sock,
     apr_status_t ss;
     apr_size_t nbytes = (apr_size_t)len;
     char *bytes;
-    apr_interval_time_t t;
 
     UNREFERENCED(o);
     TCN_ASSERT(sock != 0);
@@ -923,13 +815,9 @@ TCN_IMPLEMENT_CALL(jint, Socket, recvbt)(TCN_STDARGS, jlong sock,
     bytes  = (char *)(*e)->GetDirectBufferAddress(e, buf);
     TCN_ASSERT(bytes != NULL);
 
-    if ((ss = (*s->net->timeout_get)(s->opaque, &t)) != APR_SUCCESS)
-         return -(jint)ss;
     if ((ss = (*s->net->timeout_set)(s->opaque, J2T(timeout))) != APR_SUCCESS)
          return -(jint)ss;
     ss = (*s->net->recv)(s->opaque, bytes + offset, &nbytes);
-    /* Resore the original timeout */
-    (*s->net->timeout_set)(s->opaque, t);
 #ifdef TCN_DO_STATISTICS
     if (ss == APR_SUCCESS) {
         sp_max_recv = TCN_MAX(sp_max_recv, nbytes);
@@ -966,7 +854,6 @@ TCN_IMPLEMENT_CALL(jint, Socket, recvbbt)(TCN_STDARGS, jlong sock,
     tcn_socket_t *s = J2P(sock, tcn_socket_t *);
     apr_status_t ss;
     apr_size_t nbytes = (apr_size_t)len;
-    apr_interval_time_t t;
 
     UNREFERENCED_STDARGS;
     TCN_ASSERT(sock != 0);
@@ -974,13 +861,9 @@ TCN_IMPLEMENT_CALL(jint, Socket, recvbbt)(TCN_STDARGS, jlong sock,
     TCN_ASSERT(s->opaque != NULL);
 
 
-    if ((ss = (*s->net->timeout_get)(s->opaque, &t)) != APR_SUCCESS)
-         return -(jint)ss;
     if ((ss = (*s->net->timeout_set)(s->opaque, J2T(timeout))) != APR_SUCCESS)
          return -(jint)ss;
     ss = (*s->net->recv)(s->opaque, s->jrbbuff + offset, &nbytes);
-    /* Resore the original timeout */
-    (*s->net->timeout_set)(s->opaque, t);
 #ifdef TCN_DO_STATISTICS
     if (ss == APR_SUCCESS) {
         sp_max_recv = TCN_MAX(sp_max_recv, nbytes);
@@ -1295,51 +1178,34 @@ TCN_IMPLEMENT_CALL(jint, Socket, acceptfilter)(TCN_STDARGS,
 #endif
 }
 
-TCN_IMPLEMENT_CALL(jint, Mulicast, join)(TCN_STDARGS,
-                                         jlong sock, jlong join,
-                                         jlong iface, jlong source)
+TCN_IMPLEMENT_CALL(jint, Socket, dataSet)(TCN_STDARGS, jlong sock,
+                                          jstring key, jobject data)
 {
     tcn_socket_t *s = J2P(sock, tcn_socket_t *);
-    apr_sockaddr_t *ja = J2P(join, apr_sockaddr_t *);
-    apr_sockaddr_t *ia = J2P(iface, apr_sockaddr_t *);
-    apr_sockaddr_t *sa = J2P(source, apr_sockaddr_t *);
-    UNREFERENCED_STDARGS;
-    return (jint)apr_mcast_join(s->sock, ja, ia, sa);
-};
+    apr_status_t rv = APR_SUCCESS;
+    TCN_ALLOC_CSTRING(key);
 
-TCN_IMPLEMENT_CALL(jint, Mulicast, leave)(TCN_STDARGS,
-                                          jlong sock, jlong addr,
-                                          jlong iface, jlong source)
-{
-    tcn_socket_t *s = J2P(sock, tcn_socket_t *);
-    apr_sockaddr_t *aa = J2P(addr, apr_sockaddr_t *);
-    apr_sockaddr_t *ia = J2P(iface, apr_sockaddr_t *);
-    apr_sockaddr_t *sa = J2P(source, apr_sockaddr_t *);
-    UNREFERENCED_STDARGS;
-    return (jint)apr_mcast_leave(s->sock, aa, ia, sa);
-};
+    UNREFERENCED(o);
+    TCN_ASSERT(sock != 0);
 
-TCN_IMPLEMENT_CALL(jint, Mulicast, hops)(TCN_STDARGS,
-                                         jlong sock, jint ttl)
-{
-    tcn_socket_t *s = J2P(sock, tcn_socket_t *);
-    UNREFERENCED_STDARGS;
-    return (jint)apr_mcast_hops(s->sock, (apr_byte_t)ttl);
-};
+    rv = apr_socket_data_set(s->sock, data, J2S(key), NULL);
+    TCN_FREE_CSTRING(key);
+    return rv;
+}
 
-TCN_IMPLEMENT_CALL(jint, Mulicast, loopback)(TCN_STDARGS,
-                                             jlong sock, jboolean opt)
+TCN_IMPLEMENT_CALL(jobject, Socket, dataGet)(TCN_STDARGS, jlong socket,
+                                             jstring key)
 {
-    tcn_socket_t *s = J2P(sock, tcn_socket_t *);
-    UNREFERENCED_STDARGS;
-    return (jint)apr_mcast_loopback(s->sock, opt == JNI_TRUE ? 1 : 0);
-};
+    tcn_socket_t *s = J2P(socket, tcn_socket_t *);
+    TCN_ALLOC_CSTRING(key);
+    jobject rv = NULL;
 
-TCN_IMPLEMENT_CALL(jint, Mulicast, ointerface)(TCN_STDARGS,
-                                               jlong sock, jlong iface)
-{
-    tcn_socket_t *s = J2P(sock, tcn_socket_t *);
-    apr_sockaddr_t *ia = J2P(iface, apr_sockaddr_t *);
-    UNREFERENCED_STDARGS;
-    return (jint)apr_mcast_interface(s->sock, ia);
-};
+    UNREFERENCED(o);
+    TCN_ASSERT(sock != 0);
+
+    if (apr_socket_data_get(&rv, J2S(key), s->sock) != APR_SUCCESS) {
+        rv = NULL;
+    }
+    TCN_FREE_CSTRING(key);
+    return rv;
+}
