@@ -16,17 +16,11 @@
 /*
  *
  * @author Mladen Turk
- * @version $Revision: 393735 $, $Date: 2006-04-13 02:41:49 -0400 (Thu, 13 Apr 2006) $
+ * @version $Revision: 420070 $, $Date: 2006-07-08 02:18:55 -0500 (Sat, 08 Jul 2006) $
  */
 
-#include "apr.h"
-#include "apr_pools.h"
-#include "apr_portable.h"
 #include "tcn.h"
 
-
-
-
 TCN_IMPLEMENT_CALL(jstring, OS, defaultEncoding)(TCN_STDARGS, jlong pool)
 {
     apr_pool_t *p = J2P(pool, apr_pool_t *);
@@ -43,8 +37,3 @@ TCN_IMPLEMENT_CALL(jstring, OS, localeEncoding)(TCN_STDARGS, jlong pool)
     return AJP_TO_JSTRING(apr_os_locale_encoding(p));
 }
 
-TCN_IMPLEMENT_CALL(jlong, OS, threadCurrent)(TCN_STDARGS)
-{
-    UNREFERENCED_STDARGS;
-    return (jlong)((unsigned long)apr_os_thread_current());
-}
