@@ -16,7 +16,7 @@
 /** SSL Utilities
  *
  * @author Mladen Turk
- * @version $Revision: 300969 $, $Date: 2005-07-12 10:56:11 -0400 (Tue, 12 Jul 2005) $
+ * @version $Revision: 439960 $, $Date: 2006-09-04 02:15:58 -0500 (Mon, 04 Sep 2006) $
  */
 
 #include "tcn.h"
@@ -77,10 +77,11 @@ int SSL_password_prompt(tcn_pass_cb_t *data)
 {
     int rv = 0;
     data->password[0] = '\0';
-    if (data->cb.env && data->cb.obj) {
-        JNIEnv *e = data->cb.env;
+    if (data->cb.obj) {
+        JNIEnv *e;
         jobject  o;
         jstring  prompt = AJP_TO_JSTRING(data->prompt);
+        tcn_get_java_env(&e);
         if ((o = (*e)->CallObjectMethod(e, data->cb.obj,
                             data->cb.mid[0], prompt))) {
             TCN_ALLOC_CSTRING(o);
