@@ -19,13 +19,17 @@ package org.apache.catalina.ant.jmx;
 /**
  *
  * @author Peter Rossbach
- * @version $Revision: 303958 $ $Date: 2005-06-24 07:53:42 -0400 (Fri, 24 Jun 2005) $
+ * @version $Revision: 441172 $ $Date: 2006-09-07 13:38:07 -0500 (Thu, 07 Sep 2006) $
  * @since 5.5.10
  */
 public class Arg {
+    String name;
     String type;
     String value;
 
+    public void setName( String name) {
+        this.name=name;
+    }
     public void setType( String type) {
         this.type=type;
     }
@@ -35,11 +39,12 @@ public class Arg {
     public void addText( String text ) {
         this.value=text;
     }
-
+    public String getName() {
+        return name;
+    }
     public String getValue() {
         return value;
     }
-
     public String getType() {
         return type;
     }
