@@ -66,7 +66,7 @@ import org.apache.commons.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 345286 $ $Date: 2005-11-17 12:00:24 -0500 (Thu, 17 Nov 2005) $
+ * @version $Revision: 413538 $ $Date: 2006-06-11 16:53:07 -0500 (Sun, 11 Jun 2006) $
  */
 public class StandardWrapper
     extends ContainerBase
@@ -1308,10 +1308,6 @@ public class StandardWrapper
             }
         }
 
-        ClassLoader oldCtxClassLoader =
-            Thread.currentThread().getContextClassLoader();
-        ClassLoader classLoader = instance.getClass().getClassLoader();
-
         PrintStream out = System.out;
         if (swallowOutput) {
             SystemLogHandler.startCapture();
@@ -1322,7 +1318,6 @@ public class StandardWrapper
             instanceSupport.fireInstanceEvent
               (InstanceEvent.BEFORE_DESTROY_EVENT, instance);
 
-            Thread.currentThread().setContextClassLoader(classLoader);
             if( System.getSecurityManager() != null) {
                 SecurityUtil.doAsPrivilege("destroy",
                                            instance);
@@ -1345,8 +1340,6 @@ public class StandardWrapper
                 (sm.getString("standardWrapper.destroyException", getName()),
                  t);
         } finally {
-            // restore the context ClassLoader
-            Thread.currentThread().setContextClassLoader(oldCtxClassLoader);
             // Write captured output
             if (swallowOutput) {
                 String log = SystemLogHandler.stopCapture();
@@ -1365,7 +1358,6 @@ public class StandardWrapper
 
         if (singleThreadModel && (instancePool != null)) {
             try {
-                Thread.currentThread().setContextClassLoader(classLoader);
                 while (!instancePool.isEmpty()) {
                     if( System.getSecurityManager() != null) {
                         SecurityUtil.doAsPrivilege("destroy",
@@ -1383,10 +1375,6 @@ public class StandardWrapper
                 throw new ServletException
                     (sm.getString("standardWrapper.destroyException",
                                   getName()), t);
-            } finally {
-                // restore the context ClassLoader
-                Thread.currentThread().setContextClassLoader
-                    (oldCtxClassLoader);
             }
             instancePool = null;
             nInstances = 0;
