@@ -62,7 +62,6 @@ import org.apache.naming.JndiPermission;
 import org.apache.naming.resources.Resource;
 import org.apache.naming.resources.ResourceAttributes;
 import org.apache.tomcat.util.IntrospectionUtils;
-import org.apache.tomcat.util.compat.JdkCompat;
 
 /**
  * Specialized web application class loader.
@@ -99,7 +98,7 @@ import org.apache.tomcat.util.compat.JdkCompat;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Revision: 378251 $ $Date: 2006-02-16 09:19:13 -0500 (Thu, 16 Feb 2006) $
+ * @version $Revision: 414654 $ $Date: 2006-06-15 13:51:57 -0500 (Thu, 15 Jun 2006) $
  */
 public class WebappClassLoader
     extends URLClassLoader
@@ -139,13 +138,10 @@ public class WebappClassLoader
      * earlier versions.
      */
     protected static final String[] triggers = {
-        "javax.servlet.Servlet"                     // Servlet API
+        "javax.servlet.Servlet",                    // Servlet API
+        "javax.servlet.jsp.JspPage"                 // JSP API
     };
 
-    /** 
-     * Jdk Compatibility Support.
-     */
-    protected static JdkCompat jdkCompat = JdkCompat.getJdkCompat();
 
     /**
      * Set of package names which are not allowed to be loaded from a webapp
@@ -517,6 +513,13 @@ public class WebappClassLoader
         this.loaderDir = new File(workDir, "loader");
     }
 
+     /**
+      * Utility method for use in subclasses.
+      * Must be called before Lifecycle methods to have any effect.
+      */
+     protected void setParentClassLoader(ClassLoader pcl) {
+         parent = pcl;
+     }
 
     // ------------------------------------------------------- Reloader Methods
 
@@ -548,7 +551,7 @@ public class WebappClassLoader
         } catch (MalformedURLException e) {
             IllegalArgumentException iae = new IllegalArgumentException
                 ("Invalid repository: " + repository); 
-            jdkCompat.chainException(iae, e);
+            iae.initCause(e);
             throw iae;
         }
 
@@ -1070,7 +1073,7 @@ public class WebappClassLoader
                             && (!(name.endsWith(".class")))) {
                         // Copy binary content to the work directory if not present
                         File resourceFile = new File(loaderDir, name);
-                        url = resourceFile.toURL();
+                        url = getURI(resourceFile);
                     }
                 } catch (Exception e) {
                     // Ignore
@@ -1407,9 +1410,9 @@ public class WebappClassLoader
             URL[] urls = new URL[length];
             for (i = 0; i < length; i++) {
                 if (i < filesLength) {
-                    urls[i] = getURL(files[i]);
+                    urls[i] = getURL(files[i], true);
                 } else if (i < filesLength + jarFilesLength) {
-                    urls[i] = getURL(jarRealFiles[i - filesLength]);
+                    urls[i] = getURL(jarRealFiles[i - filesLength], true);
                 } else {
                     urls[i] = external[i - filesLength - jarFilesLength];
                 }
@@ -1837,7 +1840,7 @@ public class WebappClassLoader
         ResourceEntry entry = new ResourceEntry();
         try {
             entry.source = getURI(new File(file, path));
-            entry.codeBase = getURL(new File(file, path));
+            entry.codeBase = getURL(new File(file, path), false);
         } catch (MalformedURLException e) {
             return null;
         }   
@@ -1963,7 +1966,7 @@ public class WebappClassLoader
 
                     entry = new ResourceEntry();
                     try {
-                        entry.codeBase = getURL(jarRealFiles[i]);
+                        entry.codeBase = getURL(jarRealFiles[i], false);
                         String jarFakeUrl = getURI(jarRealFiles[i]).toString();
                         jarFakeUrl = "jar:" + jarFakeUrl + "!/" + path;
                         entry.source = new URL(jarFakeUrl);
@@ -2280,7 +2283,7 @@ public class WebappClassLoader
     /**
      * Get URL.
      */
-    protected URL getURL(File file)
+    protected URL getURL(File file, boolean encoded)
         throws MalformedURLException {
 
         File realFile = file;
@@ -2289,7 +2292,11 @@ public class WebappClassLoader
         } catch (IOException e) {
             // Ignore
         }
-        return realFile.toURL();
+        if(encoded) {
+            return getURI(realFile);
+        } else {
+            return realFile.toURL();
+        }
 
     }
 
@@ -2300,7 +2307,7 @@ public class WebappClassLoader
     protected URL getURI(File file)
         throws MalformedURLException {
 
-        return jdkCompat.getURI(file);
+        return file.toURI().toURL();
 
     }
 
