@@ -78,7 +78,7 @@ import org.apache.naming.resources.Resource;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 303352 $ $Date: 2004-10-05 13:12:52 -0400 (Tue, 05 Oct 2004) $
+ * @version $Revision: 440936 $ $Date: 2006-09-06 20:51:28 -0500 (Wed, 06 Sep 2006) $
  */
 
 public class WebappLoader
@@ -151,6 +151,12 @@ public class WebappLoader
     protected LifecycleSupport lifecycle = new LifecycleSupport(this);
 
 
+    /**
+     * If not specified, should the system class loader be used as the parent.
+     */
+    private boolean useSystemClassLoaderAsParent = true;
+
+
     /**
      * The Java class name of the ClassLoader implementation to be used.
      * This class should extend WebappClassLoader, otherwise, a different 
@@ -318,6 +324,29 @@ public class WebappLoader
     }
 
 
+    /**
+     * Return the useSystemClassLoaderAsParent flag for this Loader.
+     */
+    public boolean getUseSystemClassLoaderAsParent() {
+        
+        return (this.useSystemClassLoaderAsParent);
+        
+    }
+    
+    /**
+     * Set the useSystemClassLoaderAsParent flag for this Loader.
+     * 
+     * @param useSystemClassLoaderAsParent The useSystemClassLoaderAsParent
+     *                                     flag.
+     */
+    public void setUseSystemClassLoaderAsParent(
+            boolean useSystemClassLoaderAsParent) {
+        
+        this.useSystemClassLoaderAsParent = useSystemClassLoaderAsParent;
+        
+    }
+    
+    
     /**
      * Return the reloadable flag for this Loader.
      */
@@ -554,6 +583,7 @@ public class WebappLoader
 
     private boolean initialized=false;
 
+
     public void init() {
         initialized=true;
 
@@ -770,7 +800,11 @@ public class WebappLoader
         WebappClassLoader classLoader = null;
 
         if (parentClassLoader == null) {
-            parentClassLoader = Thread.currentThread().getContextClassLoader();
+            if (useSystemClassLoaderAsParent) {
+                parentClassLoader = ClassLoader.getSystemClassLoader();
+            } else {
+                parentClassLoader = container.getParentClassLoader();
+            }
         }
         Class[] argTypes = { ClassLoader.class };
         Object[] args = { parentClassLoader };
