@@ -42,6 +42,7 @@ import org.apache.catalina.loader.WebappLoader;
 import org.apache.catalina.realm.DataSourceRealm;
 import org.apache.catalina.realm.JDBCRealm;
 import org.apache.catalina.realm.JNDIRealm;
+import org.apache.catalina.realm.JAASRealm;
 import org.apache.catalina.realm.MemoryRealm;
 import org.apache.catalina.realm.UserDatabaseRealm;
 import org.apache.catalina.session.StandardManager;
@@ -61,7 +62,7 @@ import org.apache.commons.modeler.Registry;
  * <code>org.apache.catalina.core.StandardServer</code> component.</p>
  *
  * @author Amy Roh
- * @version $Revision: 304032 $ $Date: 2005-07-27 11:11:55 -0400 (Wed, 27 Jul 2005) $
+ * @version $Revision: 440749 $ $Date: 2006-09-06 11:06:18 -0500 (Wed, 06 Sep 2006) $
  */
 
 public class MBeanFactory extends BaseModelMBean {
@@ -404,12 +405,14 @@ public class MBeanFactory extends BaseModelMBean {
      *
      * @exception Exception if an MBean cannot be created or registered
      */
-    public String createJNDIRealm(String parent)
+    public String createJNDIRealm(String parent,String connectionURL,String connectionName,String connectionPassword)
         throws Exception {
 
          // Create a new JNDIRealm instance
         JNDIRealm realm = new JNDIRealm();
-
+				realm.setConnectionURL(connectionURL);
+				realm.setConnectionName(connectionName);
+				realm.setConnectionPassword(connectionPassword);
         // Add the new instance to its parent component
         ObjectName pname = new ObjectName(parent);
         ContainerBase containerBase = getParentContainerFromParent(pname);
@@ -426,7 +429,44 @@ public class MBeanFactory extends BaseModelMBean {
 
 
     }
+		
+  /**
+     * Create a new JAAS Realm.
+     *
+     * @param parent MBean Name of the associated parent component
+     *
+     * @exception Exception if an MBean cannot be created or registered
+     */
+    public String createJAASRealm(String parent,String appName,String userClassNames,String roleClassNames,String useContextClassLoader)
+        throws Exception {
 
+         // Create a new JAASRealm instance
+        JAASRealm realm = new JAASRealm();
+				realm.setAppName(appName);
+				realm.setUserClassNames(userClassNames);
+				realm.setRoleClassNames(roleClassNames);
+				if("true".equals(useContextClassLoader)||"TRUE".equals(useContextClassLoader)){
+						realm.setUseContextClassLoader(true);
+				}
+				else{
+						realm.setUseContextClassLoader(false);
+				}
+        // Add the new instance to its parent component
+        ObjectName pname = new ObjectName(parent);
+        ContainerBase containerBase = getParentContainerFromParent(pname);
+        // Add the new instance to its parent component
+        containerBase.setRealm(realm);
+        // Return the corresponding MBean name
+        ObjectName oname = realm.getObjectName();
+
+        if (oname != null) {
+            return (oname.toString());
+        } else {
+            return null;
+        }   
+
+
+    }
 
     /**
      * Create a new Memory Realm.
