@@ -74,7 +74,7 @@ import org.apache.catalina.security.SecurityUtil;
  * @author Craig R. McClanahan
  * @author Sean Legassick
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
- * @version $Revision: 384817 $ $Date: 2006-03-10 10:27:43 -0500 (Fri, 10 Mar 2006) $
+ * @version $Revision: 405097 $ $Date: 2006-05-08 12:22:55 -0500 (Mon, 08 May 2006) $
  */
 
 public class StandardSession
@@ -424,7 +424,7 @@ public class StandardSession
 
         if ( !isValid() ) {
             throw new IllegalStateException
-            (sm.getString("standardSession.getId.ise"));
+            (sm.getString("standardSession.getLastAccessedTime.ise"));
         }
 
         return (this.lastAccessedTime);
@@ -1230,6 +1230,10 @@ public class StandardSession
      *  invalidated session
      */
     public void setAttribute(String name, Object value) {
+        setAttribute(name,value,true);
+    }
+
+    public void setAttribute(String name, Object value, boolean notify) {
 
         // Name cannot be null
         if (name == null)
@@ -1255,7 +1259,7 @@ public class StandardSession
         HttpSessionBindingEvent event = null;
 
         // Call the valueBound() method if necessary
-        if (value instanceof HttpSessionBindingListener) {
+        if (value instanceof HttpSessionBindingListener && notify) {
             // Don't call any notification if replacing with the same value
             Object oldValue = attributes.get(name);
             if (value != oldValue) {
@@ -1274,7 +1278,7 @@ public class StandardSession
 
         // Call the valueUnbound() method if necessary
         if ((unbound != null) && (unbound != value) &&
-            (unbound instanceof HttpSessionBindingListener)) {
+            (unbound instanceof HttpSessionBindingListener)  && notify) {
             try {
                 ((HttpSessionBindingListener) unbound).valueUnbound
                     (new HttpSessionBindingEvent(getSession(), name));
@@ -1283,6 +1287,8 @@ public class StandardSession
                     (sm.getString("standardSession.bindingEvent"), t);
             }
         }
+        
+        if ( !notify ) return;
 
         // Notify interested application event listeners
         Context context = (Context) manager.getContainer();
