@@ -32,7 +32,7 @@ import org.apache.coyote.Constants;
  * 
  * @author Dan Sandberg
  * @author David Becker
- * @version $Revision: 349308 $, $Date: 2005-11-27 15:52:55 -0500 (Sun, 27 Nov 2005) $
+ * @version $Revision: 425917 $, $Date: 2006-07-26 19:34:34 -0500 (Wed, 26 Jul 2006) $
  */
 public class SSIServletExternalResolver implements SSIExternalResolver {
     protected final String VARIABLE_NAMES[] = {"AUTH_TYPE", "CONTENT_LENGTH",
@@ -533,7 +533,8 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             // a problem
             // if a truly empty file
             //were included, but not sure how else to tell.
-            if (retVal.equals("")) {
+            if (retVal.equals("") && !req.getMethod().equalsIgnoreCase(
+                    org.apache.coyote.http11.Constants.HEAD)) {
                 throw new IOException("Couldn't find file: " + path);
             }
             return retVal;
