@@ -63,7 +63,7 @@ import org.xml.sax.SAXParseException;
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 379614 $ $Date: 2006-02-21 18:02:57 -0500 (Tue, 21 Feb 2006) $
+ * @version $Revision: 394930 $ $Date: 2006-04-18 07:59:06 -0500 (Tue, 18 Apr 2006) $
  */
 
 public class ContextConfig
@@ -858,7 +858,8 @@ public class ContextConfig
             docBase = file.getCanonicalPath();
         }
         file = new File(docBase);
-
+        String origDocBase = docBase ;
+ 
         if (docBase.toLowerCase().endsWith(".war") && !file.isDirectory() && unpackWARs) {
             URL war = new URL("jar:" + (new File(docBase)).toURL() + "!/");
             String contextPath = context.getPath();
@@ -868,6 +869,9 @@ public class ContextConfig
             docBase = ExpandWar.expand(host, war, contextPath);
             file = new File(docBase);
             docBase = file.getCanonicalPath();
+            if (context instanceof StandardContext) {
+                ((StandardContext) context).setOriginalDocBase(origDocBase);
+            }
         } else {
             File docDir = new File(docBase);
             if (!docDir.exists()) {
@@ -882,6 +886,9 @@ public class ContextConfig
                         docBase = warFile.getCanonicalPath();
                     }
                 }
+                if (context instanceof StandardContext) {
+                    ((StandardContext)context).setOriginalDocBase(origDocBase);
+                }
             }
         }
 
