@@ -67,7 +67,7 @@ import org.apache.catalina.core.StandardContext;
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
  * @author Peter Rossbach
- * @version $Revision: 393447 $ $Date: 2006-04-12 08:39:05 -0400 (Wed, 12 Apr 2006) $
+ * @version $Revision: 408460 $ $Date: 2006-05-21 13:26:01 -0500 (Sun, 21 May 2006) $
  */
 
 public class DeltaManager extends ManagerBase implements Lifecycle,
@@ -198,6 +198,20 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
 
     // ------------------------------------------------------------- Properties
     
+    public static ClassLoader[] getClassLoaders(Container container) {
+        Loader loader = null;
+        ClassLoader classLoader = null;
+        if (container != null) loader = container.getLoader();
+        if (loader != null) classLoader = loader.getClassLoader();
+        else classLoader = Thread.currentThread().getContextClassLoader();
+        if ( classLoader == Thread.currentThread().getContextClassLoader() ) {
+            return new ClassLoader[] {classLoader};
+        } else {
+            return new ClassLoader[] {classLoader,Thread.currentThread().getContextClassLoader()};
+        }
+    }
+
+    
     /**
      * Return descriptive information about this Manager implementation and the
      * corresponding version number, in the format
@@ -678,19 +692,8 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
         ByteArrayInputStream fis = null;
         ReplicationStream ois = null;
         Loader loader = null;
-        ClassLoader classLoader = null;
-        //fix to be able to run the DeltaManager
-        //stand alone without a container.
-        //use the Threads context class loader
-        if (container != null)
-            loader = container.getLoader();
-        if (loader != null)
-            classLoader = loader.getClassLoader();
-        else
-            classLoader = Thread.currentThread().getContextClassLoader();
-        //end fix
         fis = new ByteArrayInputStream(data);
-        ois = new ReplicationStream(fis, classLoader);
+        ois = new ReplicationStream(fis, getClassLoaders(this.container)[0]);
         session.getDeltaRequest().readExternal(ois);
         ois.close();
         return session.getDeltaRequest();
@@ -1482,7 +1485,10 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
         if(isSendClusterDomainOnly() && !checkSenderDomain(msg,sender)) {
             return;
         }
+        ClassLoader contextLoader = Thread.currentThread().getContextClassLoader();
         try {
+            ClassLoader[] loaders = getClassLoaders(container);
+            if ( loaders != null && loaders.length > 0 ) Thread.currentThread().setContextClassLoader(loaders[0]);
             if (log.isDebugEnabled())
                 log.debug(sm.getString("deltaManager.receiveMessage.eventType",
                         getName(), msg.getEventTypeString(), sender));
@@ -1524,6 +1530,8 @@ public class DeltaManager extends ManagerBase implements Lifecycle,
         } catch (Exception x) {
             log.error(sm.getString("deltaManager.receiveMessage.error",
                     getName()), x);
+        } finally {
+            Thread.currentThread().setContextClassLoader(contextLoader);
         }
     }
 
