@@ -72,7 +72,7 @@ import org.apache.catalina.util.StringManager;
  * @author Craig R. McClanahan
  * @author Sean Legassick
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens </a>
- * @version $Revision: 384817 $ $Date: 2006-03-10 10:27:43 -0500 (Fri, 10 Mar 2006) $
+ * @version $Revision: 399358 $ $Date: 2006-05-03 12:16:15 -0500 (Wed, 03 May 2006) $
  */
 
 public class DeltaSession implements HttpSession, Session, Serializable,
@@ -438,7 +438,7 @@ public class DeltaSession implements HttpSession, Session, Serializable,
 
         if (!isValid()) {
             throw new IllegalStateException(sm
-                    .getString("standardSession.getId.ise"));
+                    .getString("standardSession.getLastAccessedTime.ise"));
         }
             
         return (this.lastAccessedTime);
@@ -600,7 +600,7 @@ public class DeltaSession implements HttpSession, Session, Serializable,
 
         if (maxInactiveInterval >= 0) {
             long timeNow = System.currentTimeMillis();
-            int timeIdle = (int) ((timeNow - lastAccessedTime) / 1000L);
+            int timeIdle = (int) ((timeNow - thisAccessedTime) / 1000L);
             if (isPrimarySession()) {
                 if(timeIdle >= maxInactiveInterval) {
                     expire(true);
