@@ -294,7 +294,7 @@ implements ClusterManager
                         long interval = session.getMaxInactiveInterval();
                         long lastaccdist = System.currentTimeMillis() -
                             session.getLastAccessWasDistributed();
-                        if ( ((interval*1000) / lastaccdist)< 3 ) {
+                        if ( interval*1000 < 3 * lastaccdist ) {
                             SessionMessage accmsg = new SessionMessageImpl(name,
                                 SessionMessage.EVT_SESSION_ACCESSED,
                                 null,
