@@ -32,7 +32,7 @@ import org.apache.catalina.util.StringManager;
  * 
  * @author Peter Rossbach
  * @author Filip Hanik
- * @version $Revision: 380229 $ $Date: 2006-02-23 16:28:29 -0500 (Thu, 23 Feb 2006) $
+ * @version $Revision: 418134 $ $Date: 2006-06-29 15:54:33 -0500 (Thu, 29 Jun 2006) $
  * @since 5.5.7
  */
 public class DataSender implements IDataSender {
@@ -259,14 +259,22 @@ public class DataSender implements IDataSender {
      * @return Returns the avg totalBytes/nrOfRequests.
      */
     public long getAvgMessageSize() {
-        return totalBytes / nrOfRequests;
+        if ( nrOfRequests > 0 ) {
+            return totalBytes / nrOfRequests;
+        } else {
+            return 0;
+        }
     }
 
     /**
      * @return Returns the avg processingTime/nrOfRequests.
      */
     public double getAvgProcessingTime() {
-        return ((double)processingTime) / nrOfRequests;
+        if ( nrOfRequests > 0 ) {
+            return ((double)processingTime) / nrOfRequests;
+        } else {
+            return 0;
+        }
     }
  
     /**
@@ -323,7 +331,11 @@ public class DataSender implements IDataSender {
      * @return Returns the avg waitAckTime/nrOfRequests.
      */
     public double getAvgWaitAckTime() {
-        return ((double)waitAckTime) / nrOfRequests;
+        if ( nrOfRequests > 0 ) {
+            return ((double)waitAckTime) / nrOfRequests;
+        } else {
+            return 0;
+        }
     }
  
     /**
