@@ -46,7 +46,7 @@ import org.apache.tomcat.util.IntrospectionUtils;
  * 
  * @author Peter Rossbach
  * @author Filip Hanik
- * @version $Revision: 380229 $ $Date: 2006-02-23 16:28:29 -0500 (Thu, 23 Feb 2006) $
+ * @version $Revision: 418134 $ $Date: 2006-06-29 15:54:33 -0500 (Thu, 29 Jun 2006) $
  */
 public class ReplicationTransmitter implements ClusterSender,IDynamicProperty {
     private static org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory
@@ -214,7 +214,11 @@ public class ReplicationTransmitter implements ClusterSender,IDynamicProperty {
      * @return Returns the avg processingTime/nrOfRequests.
      */
     public double getAvgProcessingTime() {
-        return ((double)processingTime) / nrOfRequests;
+        if ( nrOfRequests > 0 ) {
+            return ((double)processingTime) / nrOfRequests;
+        } else {
+            return 0;
+        }
     }
  
     /**
