@@ -76,7 +76,7 @@ import org.apache.tomcat.util.IntrospectionUtils;
  * @author Filip Hanik
  * @author Remy Maucherat
  * @author Peter Rossbach
- * @version $Revision: 380229 $, $Date: 2006-02-23 16:28:29 -0500 (Thu, 23 Feb 2006) $
+ * @version $Revision: 412798 $, $Date: 2006-06-08 11:57:24 -0500 (Thu, 08 Jun 2006) $
  */
 public class SimpleTcpCluster implements CatalinaCluster, Lifecycle,
         MembershipListener, LifecycleListener, IDynamicProperty {
@@ -540,26 +540,30 @@ public class SimpleTcpCluster implements CatalinaCluster, Lifecycle,
      * @see DeltaManager#start()
      */
     public synchronized Manager createManager(String name) {
-        if (log.isDebugEnabled())
-            log.debug("Creating ClusterManager for context " + name
-                    + " using class " + getManagerClassName());
+        ClassLoader oldCtxLoader = Thread.currentThread().getContextClassLoader();
         Manager manager = null;
         try {
-            manager = (Manager) getClass().getClassLoader().loadClass(
-                    getManagerClassName()).newInstance();
-        } catch (Exception x) {
-            log.error("Unable to load class for replication manager", x);
-            manager = new org.apache.catalina.cluster.session.DeltaManager();
-        } finally {
-            if(manager != null) {
-                manager.setDistributable(true);
-                if (manager instanceof ClusterManager) {
-                    ClusterManager cmanager = (ClusterManager) manager ;
-                    cmanager.setDefaultMode(true);
-                    cmanager.setName(getManagerName(name,manager));
-                    cmanager.setCluster(this);
+            Thread.currentThread().setContextClassLoader(SimpleTcpCluster.class.getClassLoader());
+            if (log.isDebugEnabled())
+                log.debug("Creating ClusterManager for context " + name + " using class " + getManagerClassName());
+            try {
+                manager = (Manager) getClass().getClassLoader().loadClass(getManagerClassName()).newInstance();
+            } catch (Exception x) {
+                log.error("Unable to load class for replication manager", x);
+                manager = new org.apache.catalina.cluster.session.DeltaManager();
+            } finally {
+                if (manager != null) {
+                    manager.setDistributable(true);
+                    if (manager instanceof ClusterManager) {
+                        ClusterManager cmanager = (ClusterManager) manager;
+                        cmanager.setDefaultMode(true);
+                        cmanager.setName(getManagerName(name, manager));
+                        cmanager.setCluster(this);
+                    }
                 }
             }
+        }finally {
+            Thread.currentThread().setContextClassLoader(oldCtxLoader);
         }
         return manager;
     }
