@@ -18,36 +18,51 @@ package org.apache.catalina.tribes;
 
 
 /**
- * Cluster Receiver Interface
+ * ChannelReceiver Interface<br>
+ * The <code>ChannelReceiver</code> interface is the data receiver component 
+ * at the bottom layer, the IO layer (for layers see the javadoc for the {@link Channel} interface).
+ * This class may optionally implement a thread pool for parallel processing of incoming messages.
  * @author Filip Hanik
  * @version $Revision: 379904 $, $Date: 2006-02-22 15:16:25 -0600 (Wed, 22 Feb 2006) $
  */
-public interface ChannelReceiver {
+public interface ChannelReceiver extends Heartbeat {
     /**
-     * Start message listing
+     * Start listening for incoming messages on the host/port
      * @throws java.io.IOException
      */
     public void start() throws java.io.IOException;
 
     /**
-     * Stop message listing 
+     * Stop listening for messages
      */
     public void stop();
 
     /**
-     * get the listing ip interface
-     * @return The host
+     * String representation of the IPv4 or IPv6 address that this host is listening
+     * to.
+     * @return the host that this receiver is listening to
      */
     public String getHost();
     
     
     /**
-     * get the listing ip port
-     * @return The port
+     * Returns the listening port
+     * @return port
      */
     public int getPort();
     
+    /**
+     * Sets the message listener to receive notification of incoming
+     * @param listener MessageListener
+     * @see MessageListener
+     */
     public void setMessageListener(MessageListener listener);
+    
+    /**
+     * Returns the message listener that is associated with this receiver
+     * @return MessageListener
+     * @see MessageListener
+     */
     public MessageListener getMessageListener();
 
 }
