@@ -16,10 +16,12 @@
 
 package org.apache.catalina.tribes;
 
-import java.io.Serializable;
+
 
 /**
- *
+ * The <code>ErrorHandler</code> class is used when sending messages
+ * that are sent asynchronously and the application still needs to get 
+ * confirmation when the message was sent successfully or when a message errored out.
  * @author Filip Hanik
  * @version 1.0
  */
@@ -27,17 +29,17 @@ public interface ErrorHandler {
     
     /**
      * Invoked if the message is dispatched asynch, and an error occurs
-     * @param x Exception
-     * @param destination Member[]
-     * @param msg Serializable
+     * @param x ChannelException - the error that happened
+     * @param id - the unique id for the message
+     * @see Channel#send(Member[], Serializable, int, ErrorHandler)
      */
-    public void handleError(Exception x, Member[] destination, Serializable msg);
+    public void handleError(ChannelException x, UniqueId id);
     
     /**
-     * Invoked when the message has been sent.
-     * @param destination Member[]
-     * @param msg Serializable
+     * Invoked when the message has been sent successfully.
+     * @param id - the unique id for the message
+     * @see Channel#send(Member[], Serializable, int, ErrorHandler)
      */
-    public void handleCompletion(Member[] destination, Serializable msg);
+    public void handleCompletion(UniqueId id);
     
 }
