@@ -15,16 +15,21 @@
  */
 package org.apache.catalina.tribes;
 
-import java.io.Serializable;
+/**
+ * 
+ * <p>Title: MessageListener</p> 
+ * 
+ * <p>Description: The listener to be registered with the ChannelReceiver, internal Tribes component</p> 
+ * 
+ * @author Filip Hanik
+ * @version 1.0
+ */
 
 public interface MessageListener {
     
     /**
-     * Receive a message from the cluster.
-     * @param msg ClusterMessage
-     * @return ClusterMessage - response to the message sent. <br>
-     * The response object may be ignored and is not required for the 
-     * implementation to send back to the sender.
+     * Receive a message from the IO components in the Channel stack
+     * @param msg ChannelMessage
      */
     public void messageReceived(ChannelMessage msg);
     
