@@ -20,10 +20,10 @@ import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.nio.ByteBuffer;
-import java.io.ObjectInputStream;
 
 /**
  * The XByteBuffer provides a dual functionality.
@@ -36,7 +36,6 @@ import java.io.ObjectInputStream;
  * Transfer package:
  * <ul>
  * <li><b>START_DATA/b> - 7 bytes - <i>FLT2002</i></li>
- * <li><b>OPTIONS</b>  - 4 bytes - message options, implementation specific</li>
  * <li><b>SIZE</b>      - 4 bytes - size of the data package</li>
  * <li><b>DATA</b>      - should be as many bytes as the prev SIZE</li>
  * <li><b>END_DATA</b>  - 7 bytes - <i>TLF2003</i></lI>
@@ -81,7 +80,10 @@ public class XByteBuffer
     protected int bufSize = 0;
     
     /**
-     * 
+     * Flag for discarding invalid packages
+     * If this flag is set to true, and append(byte[],...) is called,
+     * the data added will be inspected, and if it doesn't start with 
+     * <code>START_DATA</code> it will be thrown away.
      * 
      */
     protected boolean discard = true;
@@ -89,6 +91,7 @@ public class XByteBuffer
     /**
      * Constructs a new XByteBuffer
      * @param size - the initial size of the byte buffer
+     * @todo use a pool of byte[] for performance
      */
     public XByteBuffer(int size, boolean discard) {
         buf = new byte[size];
@@ -106,16 +109,25 @@ public class XByteBuffer
         bufSize = data.length;
         this.discard = discard;
     }
-
+    
     public int getLength() {
         return bufSize;
     }
+
+    public void setLength(int size) {
+        if ( size > buf.length ) throw new ArrayIndexOutOfBoundsException("Size is larger than existing buffer.");
+        bufSize = size;
+    }
     
     public void trim(int length) {
         if ( (bufSize - length) < 0 ) 
             throw new ArrayIndexOutOfBoundsException("Can't trim more bytes than are available. length:"+bufSize+" trim:"+length);
         bufSize -= length;
     }
+    
+    public void reset() {
+        bufSize = 0;
+    }
             
     public byte[] getBytesDirect() {
         return this.buf;
@@ -248,7 +260,11 @@ public class XByteBuffer
      * within the buffer
      * @return - true if a complete package (header,compress,size,data,footer) exists within the buffer
      */
-    public int countPackages()
+    public int countPackages() {
+        return countPackages(false);
+    }
+
+    public int countPackages(boolean first)
     {
         int cnt = 0;
         int pos = START_DATA.length;
@@ -276,6 +292,8 @@ public class XByteBuffer
             //reset the values
             start = pos + END_DATA.length;
             pos = start + START_DATA.length;
+            //we only want to verify that we have at least one package
+            if ( first ) break;
         }
         return cnt;
     }
@@ -285,7 +303,7 @@ public class XByteBuffer
      * @return - true if a complete package (header,options,size,data,footer) exists within the buffer
      */
     public boolean doesPackageExist()  {
-        return (countPackages()>0);
+        return (countPackages(true)>0);
     }
 
     /**
@@ -294,24 +312,27 @@ public class XByteBuffer
      * @param clearFromBuffer - if true, the package will be removed from the byte buffer
      * @return - returns the actual message bytes (header, compress,size and footer not included).
      */
-    public byte[] extractDataPackage(boolean clearFromBuffer) {
-        int psize = countPackages();
-        if (psize == 0) throw new java.lang.IllegalStateException("No package exists in XByteBuffer");
+    public XByteBuffer extractDataPackage(boolean clearFromBuffer) {
+        int psize = countPackages(true);
+        if (psize == 0) {
+            throw new java.lang.IllegalStateException("No package exists in XByteBuffer");
+        }
         int size = toInt(buf, START_DATA.length);
-        byte[] data = new byte[size];
-        System.arraycopy(buf, START_DATA.length + 4, data, 0, size);
+        XByteBuffer xbuf = BufferPool.getBufferPool().getBuffer(size,false);
+        xbuf.setLength(size);
+        System.arraycopy(buf, START_DATA.length + 4, xbuf.getBytesDirect(), 0, size);
         if (clearFromBuffer) {
             int totalsize = START_DATA.length + 4 + size + END_DATA.length;
             bufSize = bufSize - totalsize;
             System.arraycopy(buf, totalsize, buf, 0, bufSize);
         }
-        return data;
+        return xbuf;
 
     }
     
-    public ClusterData extractPackage(boolean clearFromBuffer) throws java.io.IOException {
-        byte[] data = extractDataPackage(clearFromBuffer);
-        ClusterData cdata = ClusterData.getDataFromPackage(data);
+    public ChannelData extractPackage(boolean clearFromBuffer) throws java.io.IOException {
+        XByteBuffer xbuf = extractDataPackage(clearFromBuffer);
+        ChannelData cdata = ChannelData.getDataFromPackage(xbuf);
         return cdata;
     }
     
@@ -319,13 +340,39 @@ public class XByteBuffer
      * Creates a complete data package
      * @param indata - the message data to be contained within the package
      * @param compressed - compression flag for the indata buffer
-     * @return - a full package (header,compress,size,data,footer)
+     * @return - a full package (header,size,data,footer)
      * 
      */
-    public static byte[] createDataPackage(ClusterData cdata) {
-        return createDataPackage(cdata.getDataPackage());
+    public static byte[] createDataPackage(ChannelData cdata) {
+//        return createDataPackage(cdata.getDataPackage());
+        //avoid one extra byte array creation
+        int dlength = cdata.getDataPackageLength();
+        int length = getDataPackageLength(dlength);
+        byte[] data = new byte[length];
+        int offset = 0;
+        System.arraycopy(START_DATA, 0, data, offset, START_DATA.length);
+        offset += START_DATA.length;
+        toBytes(dlength,data, START_DATA.length);
+        offset += 4;
+        cdata.getDataPackage(data,offset);
+        offset += dlength;
+        System.arraycopy(END_DATA, 0, data, offset, END_DATA.length);
+        offset += END_DATA.length;
+        return data;
     }
     
+    public static byte[] createDataPackage(byte[] data, int doff, int dlength, byte[] buffer, int bufoff) {
+        if ( (buffer.length-bufoff) > getDataPackageLength(dlength) ) {
+            throw new ArrayIndexOutOfBoundsException("Unable to create data package, buffer is too small.");
+        }
+        System.arraycopy(START_DATA, 0, buffer, bufoff, START_DATA.length);
+        toBytes(data.length,buffer, bufoff+START_DATA.length);
+        System.arraycopy(data, doff, buffer, bufoff+START_DATA.length + 4, dlength);
+        System.arraycopy(END_DATA, 0, buffer, bufoff+START_DATA.length + 4 + data.length, END_DATA.length);
+        return buffer;
+    }
+
+    
     public static int getDataPackageLength(int datalength) {
         int length = 
             START_DATA.length + //header length
@@ -342,22 +389,13 @@ public class XByteBuffer
         return createDataPackage(data,0,data.length,result,0);
     }
     
-    public static byte[] createDataPackage(byte[] data, int doff, int dlength, byte[] buffer, int bufoff) {
-        if ( (buffer.length-bufoff) > getDataPackageLength(dlength) ) {
-            throw new ArrayIndexOutOfBoundsException("Unable to create data package, buffer is too small.");
-        }
-        System.arraycopy(START_DATA, 0, buffer, bufoff, START_DATA.length);
-        toBytes(data.length,buffer, bufoff+START_DATA.length);
-        System.arraycopy(data, doff, buffer, bufoff+START_DATA.length + 4, dlength);
-        System.arraycopy(END_DATA, 0, buffer, bufoff+START_DATA.length + 4 + data.length, END_DATA.length);
-        return buffer;
-    }
+    
 
-    public static void fillDataPackage(byte[] data, int doff, int dlength, XByteBuffer buf) {
-        int pkglen = getDataPackageLength(dlength);
-        if ( buf.getCapacity() <  pkglen ) buf.expand(pkglen);
-        createDataPackage(data,doff,dlength,buf.getBytesDirect(),buf.getLength());
-    }
+//    public static void fillDataPackage(byte[] data, int doff, int dlength, XByteBuffer buf) {
+//        int pkglen = getDataPackageLength(dlength);
+//        if ( buf.getCapacity() <  pkglen ) buf.expand(pkglen);
+//        createDataPackage(data,doff,dlength,buf.getBytesDirect(),buf.getLength());
+//    }
 
     /**
      * Convert four bytes to an int
@@ -562,5 +600,12 @@ public class XByteBuffer
         return data;
     }
 
-    
+    public void setDiscard(boolean discard) {
+        this.discard = discard;
+    }
+
+    public boolean getDiscard() {
+        return discard;
+    }
+
 }
