@@ -17,6 +17,8 @@
 
 package org.apache.catalina.tribes.membership;
 
+import org.apache.catalina.tribes.util.Arrays;
+
 
 /**
  * Manifest constants for the <code>org.apache.catalina.tribes.membership</code>
@@ -24,10 +26,14 @@ package org.apache.catalina.tribes.membership;
  *
  * @author Peter Rossbach
  * @version $Revision: 303950 $ $Date: 2005-06-09 15:38:30 -0500 (Thu, 09 Jun 2005) $
+ * @author Filip Hanik
  */
 
 public class Constants {
 
     public static final String Package = "org.apache.catalina.tribes.membership";
-
+    public static void main(String[] args) throws Exception {
+        System.out.println(Arrays.toString("TRIBES-B".getBytes()));
+        System.out.println(Arrays.toString("TRIBES-E".getBytes()));
+    }
 }
