@@ -22,7 +22,7 @@ import org.apache.catalina.tribes.io.XByteBuffer;
 /**
  * Manifest constants for the <code>org.apache.catalina.tribes.transport</code>
  * package.
- *
+ * @author Filip Hanik
  * @author Peter Rossbach
  * @version $Revision: 303753 $ $Date: 2005-03-14 15:24:30 -0600 (Mon, 14 Mar 2005) $
  */
@@ -31,7 +31,12 @@ public class Constants {
 
     public static final String Package = "org.apache.catalina.tribes.transport";
     
+    /*
+     * Do not change any of these values!
+     */
     public static final byte[] ACK_DATA = new byte[] {6, 2, 3};
+    public static final byte[] FAIL_ACK_DATA = new byte[] {11, 0, 5};
     public static final byte[] ACK_COMMAND = XByteBuffer.createDataPackage(ACK_DATA);
+    public static final byte[] FAIL_ACK_COMMAND = XByteBuffer.createDataPackage(FAIL_ACK_DATA);
 
 }
