@@ -931,16 +931,8 @@ public class DeltaManager extends ClusterManagerBase{
     protected Member findSessionMasterMember() {
         Member mbr = null;
         Member mbrs[] = cluster.getMembers();
-        String localMemberDomain = cluster.getLocalMember().getDomain();
-        if(isSendClusterDomainOnly()) {
-            for (int i = 0; mbr == null && i < mbrs.length; i++) {
-                Member member = mbrs[i];
-                if(localMemberDomain.equals(member.getDomain())) mbr = member ;
-            }
-        } else {
-            if(mbrs.length != 0 ) mbr = mbrs[0];
-        }
-        if(mbr == null && log.isWarnEnabled()) log.warn(sm.getString("deltaManager.noMasterMember",getName(), localMemberDomain));
+        if(mbrs.length != 0 ) mbr = mbrs[0];
+        if(mbr == null && log.isWarnEnabled()) log.warn(sm.getString("deltaManager.noMasterMember",getName(), ""));
         if(mbr != null && log.isDebugEnabled()) log.warn(sm.getString("deltaManager.foundMasterMember",getName(), mbr));
         return mbr;
     }
@@ -1263,15 +1255,14 @@ public class DeltaManager extends ClusterManagerBase{
      * Test that sender and local domain is the same
      */
     protected boolean checkSenderDomain(SessionMessage msg,Member sender) {
-        String localMemberDomain = cluster.getLocalMember().getDomain();
-        boolean sameDomain= localMemberDomain.equals(sender.getDomain());
+        boolean sameDomain= true;
         if (!sameDomain && log.isWarnEnabled()) {
                 log.warn(sm.getString("deltaManager.receiveMessage.fromWrongDomain",
                          new Object[] {getName(), 
                          msg.getEventTypeString(), 
                          sender,
-                         sender.getDomain(),
-                         localMemberDomain }));
+                         "",
+                         "" }));
         }
         return sameDomain ;
     }
@@ -1291,7 +1282,11 @@ public class DeltaManager extends ClusterManagerBase{
         if(isSendClusterDomainOnly() && !checkSenderDomain(msg,sender)) {
             return;
         }
+        ClassLoader contextLoader = Thread.currentThread().getContextClassLoader();
         try {
+            
+            ClassLoader[] loaders = getClassLoaders();
+            if ( loaders != null && loaders.length > 0) Thread.currentThread().setContextClassLoader(loaders[0]);
             if (log.isDebugEnabled()) log.debug(sm.getString("deltaManager.receiveMessage.eventType",getName(), msg.getEventTypeString(), sender));
  
             switch (msg.getEventType()) {
@@ -1330,6 +1325,8 @@ public class DeltaManager extends ClusterManagerBase{
             } //switch
         } catch (Exception x) {
             log.error(sm.getString("deltaManager.receiveMessage.error",getName()), x);
+        } finally {
+            Thread.currentThread().setContextClassLoader(contextLoader);
         }
     }
 
