@@ -340,8 +340,9 @@ public class SimpleTcpCluster
     /**
      * has members
      */
+    protected boolean hasMembers = false;
     public boolean hasMembers() {
-        return channel.hasMembers();
+        return hasMembers;
     }
     
     /**
@@ -476,12 +477,15 @@ public class SimpleTcpCluster
     public synchronized Manager createManager(String name) {
         if (log.isDebugEnabled()) log.debug("Creating ClusterManager for context " + name + " using class " + getManagerClassName());
         Manager manager = null;
+        ClassLoader oldCtxLoader = Thread.currentThread().getContextClassLoader();
         try {
+            Thread.currentThread().setContextClassLoader(SimpleTcpCluster.class.getClassLoader());
             manager = (Manager) getClass().getClassLoader().loadClass(getManagerClassName()).newInstance();
         } catch (Exception x) {
             log.error("Unable to load class for replication manager", x);
             manager = new org.apache.catalina.ha.session.DeltaManager();
         } finally {
+            Thread.currentThread().setContextClassLoader(oldCtxLoader);
             if(manager != null) {
                 manager.setDistributable(true);
                 if (manager instanceof ClusterManager) {
@@ -649,6 +653,7 @@ public class SimpleTcpCluster
         try {
             if ( clusterDeployer != null ) clusterDeployer.setCluster(this);
             this.registerClusterValve();
+            if ( channel == null ) channel = new GroupChannel();
             channel.addMembershipListener(this);
             channel.addChannelListener(this);
             channel.start(channel.DEFAULT);
@@ -794,6 +799,7 @@ public class SimpleTcpCluster
      */
     public void memberAdded(Member member) {
         try {
+            hasMembers = channel.hasMembers();
             if (log.isInfoEnabled()) log.info("Replication member added:" + member);
             // Notify our interested LifecycleListeners
             lifecycle.fireLifecycleEvent(BEFORE_MEMBERREGISTER_EVENT, member);
@@ -811,8 +817,9 @@ public class SimpleTcpCluster
      * @see org.apache.catalina.ha.MembershipListener#memberDisappeared(org.apache.catalina.ha.Member)
      */
     public void memberDisappeared(Member member) {
-        if (log.isInfoEnabled()) log.info("Received member disappeared:" + member);
         try {
+            hasMembers = channel.hasMembers();            
+            if (log.isInfoEnabled()) log.info("Received member disappeared:" + member);
             // Notify our interested LifecycleListeners
             lifecycle.fireLifecycleEvent(BEFORE_MEMBERUNREGISTER_EVENT, member);
             // Notify our interested LifecycleListeners
@@ -820,7 +827,6 @@ public class SimpleTcpCluster
         } catch (Exception x) {
             log.error("Unable remove cluster node from replication system.", x);
         }
-
     }
 
     // --------------------------------------------------------- receiver
