@@ -34,7 +34,7 @@ import javax.management.ObjectName;
  *
  * @author Craig R. McClanahan
  * @author Amy Roh
- * @version $Revision: 302986 $ $Date: 2004-06-27 22:14:52 -0400 (Sun, 27 Jun 2004) $
+ * @version $Revision: 440953 $ $Date: 2006-09-06 21:47:31 -0500 (Wed, 06 Sep 2006) $
  */
 
 public class Lists {
@@ -532,9 +532,8 @@ public class Lists {
         (MBeanServer mbserver, String domain, HttpServletRequest request)
         throws Exception {
 
-        String adminDomain = TomcatTreeBuilder.DEFAULT_DOMAIN;
         // Get the admin app's service name
-        StringBuffer sb = new StringBuffer(adminDomain);
+        StringBuffer sb = new StringBuffer(domain);
         sb.append(":type=Service,*");
         ObjectName search = new ObjectName(sb.toString());
         Iterator names = mbserver.queryNames(search, null).iterator();
@@ -561,8 +560,7 @@ public class Lists {
         throws Exception {
         
         // Get the admin app's host name
-        String adminDomain = TomcatTreeBuilder.DEFAULT_DOMAIN;
-        StringBuffer sb = new StringBuffer(adminDomain);
+        StringBuffer sb = new StringBuffer(domain);
         sb.append(":j2eeType=WebModule,*"); 
         ObjectName search = new ObjectName(sb.toString());
         Iterator names = mbserver.queryNames(search, null).iterator();
